function get_offset_fit, cen, ref, x, y, xx, yy
  off = cen-ref
  irr = fltarr(3, n_elements(cen))
  irr[0,*] = x
  irr[1,*] = y
  irr[2,*] = off
  offset = sfit(irr, 2, /irregular, max_degree=2, kx=kx)
  return, kx[0] + kx[1]*yy + kx[2]*yy^2 + kx[3]*xx + kx[4]*xx*yy + kx[5]*xx^2
end

pro get_offsets, im, origx, origy, xx, yy, xoffset, yoffset, p
  cntrd, im, origx, origy, xcen, ycen, 10, /silent
  good = where(xcen ne -1)

  if good[0] eq -1 then begin
    print, 'Warning! Unable to find stars in ' + files[i]
    xoffset=-1
    yoffset=-1
  endif

  idxgood = where(xcen[good] le !values.f_infinity)

  if idxgood[0] eq -1 then begin
    print, 'Warning! Unable to find stars in ' + files[i]
    xoffset=-1
    yoffset=-1
  endif

  idxidxgood = where(ycen[good[idxgood]] le !values.f_infinity)
  if idxidxgood[0] eq -1 then begin
    print, 'Warning! Unable to find stars in ' + files[i]
    xoffset=-1
    yoffset=-1
  endif

  p = float(n_elements(idxidxgood))/n_elements(origx)
  print, 'Found ' + strtrim(p*100,2) + '% of stars'
  xcen = xcen[good[idxgood[idxidxgood]]]
  ycen = ycen[good[idxgood[idxidxgood]]]
  x = origx[good[idxgood[idxidxgood]]]
  y = origy[good[idxgood[idxidxgood]]]
  xoffset = get_offset_fit(xcen, x, x, y, xx, yy)
  yoffset = get_offset_fit(ycen, y, x, y, xx, yy)

  sz = size(im)

  tvscl, rescale(rebin(im, sz[1]/2, sz[2]/2), 0, 0.999)
  tvcircle, 5, xcen/2, ycen/2
end

pro resizeim, im
  sz = size(im)
  xsize = sz[2] - (sz[2] mod 2)
  ysize = sz[3] - (sz[3] mod 2)

  im = im[*, 0:xsize-1, 0:ysize-1]
end

pro getpoint, im, xcen, ycen
  cursor, x, y, /device, /up
  x *= 2
  y *= 2
  cntrd, im, x, y, xcen, ycen, 10
  tvcircle, 5, xcen/2, ycen/2
end

function computeAngle, xcen1, ycen1, xcen2, ycen2

  angle = atan((ycen2-ycen1)/(xcen2-xcen1))
;  if (xcen1 lt xcen2) and (ycen1 lt ycen2) then angle = angle
;  if (xcen1 lt xcen2) and (ycen1 gt ycen2) then angle = angle + 2*!pi
;  if (xcen1 gt xcen2) and (ycen1 lt ycen2) then angle = angle + !pi
;  if (xcen1 gt xcen2) and (ycen1 gt ycen2) then angle = angle + !pi

  return, angle

end

pro alignImages, im1name, im2name

  im1 = readfits(im1name)
  im2 = readfits(im2name)

  resizeIm, im1
  resizeIm, im2

  tot1 = rescale(total(im1, 1), 0.1, 0.999)
  tot2 = rescale(total(im2, 1), 0.1, 0.999)

  sz = size(im1)
  window, 0, xsize=sz[2]/2, ysize=sz[3]/2
  tvscl, rebin(tot1, sz[2]/2, sz[3]/2)
  getPoint, tot1, xcen1, ycen1
  getPoint, tot1, xcen2, ycen2

  angle1 = computeangle(xcen1, ycen1, xcen2, ycen2)

  window, 1, xsize=sz[2]/2, ysize=sz[3]/2
  tvscl, rebin(tot2, sz[2]/2, sz[3]/2)
  getPoint, tot2, xcen3, ycen3
  getPoint, tot2, xcen4, ycen4

  angle2 = computeangle(xcen3, ycen3, xcen4, ycen4)
  
  dx = xcen1-xcen3
  dy = ycen1-ycen3

  print, angle1, angle2, dx, dy

  angle = angle2 - angle1
  angle = angle * 180.0/!pi

  zsize = sz[1]
  xsize = sz[2]
  ysize = sz[3]
  xx = findgen(xsize) # replicate(1.0, ysize)
  yy = replicate(1.0, xsize) # findgen(ysize)

  for i=0, sz[1]-1 do begin
    im2[i,*,*] = rot(reform(im2[i,*,*]), angle, cubic=-0.5)
;    im2[i,*,*] = interpolate(rot(reform(im2[i,*,*]), angle, 1.0, xcen3, ycen3, cubic=-0.5), xx-dx, yy-dy)
  endfor

  tot2 = rescale(total(im2, 1), 0.99, 1.0)

  crunchx1 = total(tot1, 2)
  crunchy1 = total(tot1, 1)
  crunchx2 = total(tot2, 2)
  crunchy2 = total(tot2, 1)

  lagx = findgen(sz[2])-(sz[2]/2)
  lagy = findgen(sz[3])-(sz[3]/2)

  cc = c_correlate(crunchx1, crunchx2, lagx)
  max = max(cc, idx)
  idx = (idx < (xsize-11)) > 10
  fit = poly_fit(lagx[idx-10:idx+10], cc[idx-10:idx+10], 2)
  dx = -0.5*fit[1]/fit[2]

  cc = c_correlate(crunchy1, crunchy2, lagy)
  max = max(cc, idx)
  idx = (idx < (ysize-11)) > 10
  fit = poly_fit(lagy[idx-10:idx+10], cc[idx-10:idx+10], 2)
  dy = -0.5*fit[1]/fit[2]

  for j=0, sz[1]-1 do begin
;     im1[j,*,*] += interpolate(im2[j,*,*], findgen(xsize)+dx, findgen(ysize)+dy, /grid, cubic=-0.6)
    im2[j,*,*] = interpolate(im2[j,*,*], findgen(xsize)+dx, findgen(ysize)+dy, /grid, cubic=-0.6)
  endfor

  xoffsettotal = fltarr(xsize, ysize)
  yoffsettotal = fltarr(xsize, ysize)

  tot2 = total(im2,1)
  p=0

  threshold = tot1[(sort(tot1))[0.9*n_elements(tot1)]]
  print, threshold
  find, tot1, x, y, flux, sharp, round, threshold, 5, [-1,1], [-1,1], /silent
  xx = findgen(xsize) # replicate(1.0, ysize)
  yy = replicate(1.0, xsize) # findgen(ysize)

  while p lt 0.8 do begin
    get_offsets, tot2, x, y, xx, yy, xoffset, yoffset, p

    xoffsettotal += xoffset
    yoffsettotal += yoffset

    if p ge 0.8 then continue

    tot2 = interpolate(tot2, xx+xoffset, yy+yoffset)
  endwhile

  for j=0, zsize-1 do begin
    im1[j,*,*] += interpolate(im2[j,*,*], xx+xoffsettotal, yy+yoffsettotal)
  endfor

  writefits, 'total-out.fits', im1

end
