function automaticallyAdjustRv, rv, mono, referenceMono, isl, pixRadius
COMPILE_OPT IDL2

  if n_elements(pixRadius) eq 0 then pixRadius = 10
  if n_elements(isl) eq 0 then isl = 300

  sz = size(rv)
  xsize = sz[1]
  ysize = sz[2]

  toProcess = where(mono lt referenceMono)
  pos = array_indices(mono, toProcess)

  distance = findgen(2*pixRadius+1, 2*pixRadius+1)
  distancex = float(floor(distance / (2*pixRadius+1))) - pixRadius
  distancey = distance mod (2*pixRadius+1) - pixRadius

  distance = sqrt(distancex*distancex + distancey*distancey)
  distance = -(distance - max(distance))
  distance = distance / max(distance)

  ; Distance: 1 au centre, 0 en bordure

  mono = mono*mono
  referenceMono = referenceMono * referenceMono

  for i=0, n_elements(toProcess)-1 do begin

    x = pos[0, i]
    y = pos[1, i]

    xmin = (x - pixRadius)>0
    xmax = (x + pixRadius)<(xsize-1)
    ymin = (y - pixRadius)>0
    ymax = (y + pixRadius)<(ysize-1)

    rvs = rv[xmin:xmax, ymin:ymax]
    monos = mono[xmin:xmax, ymin:ymax]
    dt = distance[xmin+pixRadius-x:xmax-x+pixRadius, ymin+pixRadius-y:ymax-y+pixRadius]

    idx = where(monos ge referenceMono)

    if idx[0] eq -1 then continue

    meanRv = total(rvs[idx]*monos[idx]*distance[idx])/total(monos[idx]*distance[idx])

    if (meanRv gt rv[x,y] + (isl/10.0)) or (meanRv lt rv[x,y] - (isl/10.0)) then begin
      rv[x,y] = meanRv
    endif
  endfor

  return, rv

end
