

function localMaximum, data, offset1, offset2

  fit = poly_fit(findgen(offset2-offset1+1)+offset1, data[offset1:offset2], 2)

  return, -0.5*fit[1]/fit[2]
end


pro calibrateAbsoluteLambdaAll

  spawn, 'find ./* -type d -maxdepth 0', dirs

  if dirs[0] eq '' then return

  for i=0, n_elements(dirs)-1 do begin
    spawn, 'find ' + dirs[i] + '/lambda*.ad3 | grep -v OH', files
    if files[0] eq '' then continue

    ; On utilise le fichier le plus recent pour calibrer
    ; C'est probablement le fichier le mieux aligne
    info = file_info(files)
    s = sort(info.mtime)

    print, 'Calibrating using ' + files[s[n_elements(s)-1]]

    calibrateabsolutelambda, files[s[n_elements(s)-1]]

  endfor


;  for i=0, n_elements(files)-1 do begin
;    print, 'Going with file ' + files[i]
;    calibrateabsolutelambda, files[i]
;  endfor

end

pro calibrateabsolutelambda, lambdafile

  cube = readad3(lambdafile, trailer)
  sz = size(cube)

  if n_elements(strsplit(lambdafile, '/')) eq 1 then begin
    dir = './'
  endif else begin
    info = strsplit(lambdafile, '/')
    dir = string((byte(lambdafile))[0:info[n_elements(info)-1]-1])
  endelse
  
  validCalib = readfits(dir + '/cal_valid.fits')
  if (n_elements(validCalib) eq 1) and (validCalib[0] eq -1) then begin
    region = {sky_zone}
    region.x1 = 0
    region.x2 = sz[2]-1
    region.y1 = 0
    region.y2 = sz[3]-1
    region = medianRegion(cube, region, binSize=10)
  endif else begin
    cuber = reform(cube, [sz[1], sz[2]*sz[3]])
    region = medianRegion(cuber[*, validCalib])
    cuber = 0
  endelse

  xi1 = trailer.xi1
  xl1 = trailer.xl1

  step = xi1 / float(sz[1])

  oh = getEmissionOh()
  ohlambda = oh[0,*]
  ohemission = oh[1,*]

  idx = where((ohlambda ge (xl1 - xi1)) and (ohlambda lt (xl1+2.0*xi1)))
  
  ; Il faut plotter 2 fois le spectre de la galaxie
  ; pour eviter les problemes de /data...
  window, 1, title='Galaxy spectrum'
  plot, region, xrange=[0, sz[1]-1], xstyle=1

  window, 0, title='OH spectrum'
  plot, ohlambda[idx], ohemission[idx], xrange=[ohlambda[idx[0]], ohlambda[idx[n_elements(idx)-1]]], xstyle=1, yrange=[0,1], ystyle=1
  idx1 = where((ohlambda ge (xl1 - xi1)) and (ohlambda lt xl1))
  oplot, ohlambda[idx1] + xi1, ohemission[idx1], psym=3
  idx1 = where((ohlambda ge (xl1 + xi1)) and (ohlambda lt (xl1+2.0*xi1)))
  oplot, ohlambda[idx1] - xi1, ohemission[idx1], psym=3
  oplot, [xl1, xl1], [0,1]
  oplot, [xl1+xi1, xl1+xi1], [0,1]

  tvset, 0
  print, ''
  print, 'Click on the left of the spectrum reference line'
  cursor, x, y, /up, /data
  m = min(abs(ohlambda[idx]-x), pos1)
  oplot, [ohlambda[idx[pos1]]], [ohemission[idx[pos1]]], psym=1

  print, 'Click on the right of the spectrum reference line'
  cursor, x, y, /up, /data
  m = min(abs(ohlambda[idx]-x), pos2)
  oplot, [ohlambda[idx[pos2]]], [ohemission[idx[pos2]]], psym=1

  posreal = localMaximum(ohemission[idx], pos1, pos2)
  val = ohlambda[idx[floor(posreal)]] + (ohlambda[idx[ceil(posreal)]] - ohlambda[idx[floor(posreal)]]) * (posreal - floor(posreal))
  print, val
  oplot, [val, val], [0,1]

  tvset, 1
  plot, region, xrange=[0, sz[1]-1], xstyle=1
  
  print, ''
  print, 'Click on the left of the galaxy reference line'
  cursor, x, y, /up, /data
  pos1 = round(x)
  oplot, [pos1], [region[pos1]], psym=1

  print, 'Click on the right of the galaxy reference line'
  cursor, x, y, /up, /data
  pos2 = round(x)
  oplot, [pos2], [region[pos2]], psym=1

  posgalaxy = localMaximum(region, pos1, pos2)
 
;  b = baryc(region, float(pos1))
;  posgalaxy = b[0]

  oplot, [posgalaxy, posgalaxy], minmax(region)

  print, ''
  yesno = ''
  read, yesno, prompt= 'Do we have to resolve interfringe on the spectrum reference line (y/n)? '
  yesno = strlowcase(yesno)

  print, ''
  print, 'OH spectrum reference line is at ' + strtrim(val,2) + ' angstrom'
  print, 'This line is seen at channel ' + strtrim(posgalaxy,2) + ' in the data cube'

  if yesno eq 'y' then begin
    if val gt (xi1 + xl1) then begin
      ; Interfringe high
      val = val - xi1
    endif else begin
      val = val + xi1
    endelse
  endif

  first = val - posgalaxy*step

  print, 'Data cube first channel wavelength should then be at ' + strtrim(first,2) + ' angstrom instead of ' + strtrim(xl1,2)

  print, ''
  read, yesno, prompt='Do you want to modify ad3 file? '
  yesno = strlowcase(yesno)

  trailer.xl1 = first
  if yesno eq 'y' then begin
    print, 'Writing new value in ad3 trailer... please modify .adp accordingly.'
    toto = writeAdTrailer(lambdafile, trailer)
  endif

  print, ''
  read, yesno, prompt='Do you want to propagate this new information to all lambda*.ad3 files in this directory? '
  yesno = strlowcase(yesno)

  if yesno eq 'y' then begin
    if n_elements(strsplit(lambdafile, '/')) eq 1 then begin
      dir = './'
    endif else begin
      info = strsplit(lambdafile, '/')
      dir = string((byte(lambdafile))[0:info[n_elements(info)-1]-1])
    endelse

    files = findfile(dir + 'lambda*.ad3')
    filesgz = findfile(dir + 'lambda*.ad3.gz')
    if filesgz[0] ne '' then files = [files, filesgz]
    print, 'We will modify these files:'
    print, files
    read, yesno, prompt='Is this ok? '
    yesno = strlowcase(yesno)

    if yesno eq 'y' then begin
      for i=0, n_elements(files)-1 do begin
        tr = readAdTrailer(files[i])
        tr.xl1 = trailer.xl1
        toto = writeAdTrailer(files[i], tr)
        wait,2 ; Pour eviter que tous les fichiers aient le meme timestamp
      endfor
    endif
  endif

end
