;######################################################################
;
; Copyright (C) 1996-2002, Michele Cappellari
; E-mail: cappellari@strw.leidenuniv.nl
;
; This software is provided as is without any warranty whatsoever.
; Permission to use, copy and distribute unmodified copies for
; non-commercial purposes is granted. Permission to modify for
; personal or internal use is granted, provided this copyright
; and disclaimer are included unchanged at the beginning of
; the file. All other rights are reserved.
;
;######################################################################
FUNCTION Columns_Read, NomeFile, columns, rows
COMPILE_OPT IDL2
;
; Returns an array containing the columns COLUMNS, starting from
; row ROWS[0] until row ROWS[1], read from the text file NomeFile.
; COLUMNS = [c1,c2,...,cn] list of the columns to read.
; ROWS = [start,end] starting and ending rows.
; If ROWS is a scalar it specifies the starting row and
;   the column is read to the end of the file.
; If ROWS is not given the whole column is read, from
;   the beginning to the end of the file.
;
;-LIMITATIONS: the rows in the file need to contain only numbers
; and not also text BEFORE the last column to read.
;
;-EXAMPLE:
; In this example we read columns 1 and 3 from the text file 'dati.txt'
;
; a = Columns_Read('dati.txt',[1,3])   ; reads from the 1st to the last row
; a = Columns_Read('dati.txt',[1,3],2) ; reads from the 2nd to the last row
; a = Columns_Read('dati.txt',[1,3],[2,98]) ; reads from the 2nd to the 98th row
; PLOT, a[*,0], a[*,1]
;
; HISTORY:
;   V1.0: Michele Cappellari, Vicenza, 8 maggio 1996
;   V1.1: Added possibility not to specify any ROWS parameter.
;         MC, Leiden, 27 NOV 2002
;
ON_ERROR, 2

stringa = ''
line = DBLARR(MAX(columns), /NOZERO)
OPENR, Unit, NomeFile, /GET_LUN

IF N_PARAMS() GE 3 THEN $
    FOR j=1, rows[0]-1 DO READF, Unit, stringa ; skip first lines

IF N_ELEMENTS(rows) EQ 2 THEN BEGIN
    dati = FLTARR(rows[1]-rows[0]+1, N_ELEMENTS(columns), /NOZERO)
    FOR j=0, rows[1]-rows[0] DO BEGIN
        READF, Unit, line
        dati[j,*] = line[columns-1]
    ENDFOR
ENDIF ELSE BEGIN
    READF, Unit, line
    dati = line[columns-1]
    WHILE NOT EOF(Unit) DO BEGIN
        READF, Unit, line
        dati = [ [dati], [line[columns-1]] ]
    ENDWHILE
    dati = TRANSPOSE(dati)
ENDELSE
FREE_LUN, Unit

print, 'Read array size:', size(dati,/DIMENSIONS)

RETURN, dati
END
;--------------------------------------------------------------------------
