function add_zeros, strings, width

  temp = strings

  for i=0, n_elements(temp)-1 do begin
    while strlen(temp[i]) lt width do begin
      temp[i] = '0' + temp[i]
    endwhile
  endfor

  return, temp

end

function file_info_gz, filename
  info = file_info(filename)
  if info.exists eq 0 then info = file_info(filename + '.gz')

  return, info
end

function file_test_gz, filename, _extra = e
  test = file_test(filename, _extra = e)
  if test eq 0 then test = file_test(filename + '.gz', _extra = e)
  return, test
end

;pro applyInterfringe, dir, options, trailer, p
;COMPILE_OPT, IDL2, HIDDEN
;
;  files = file_search(dir + '/rv' + options + '*.ad2')
;
;
;
;  for i=0, n_elements(files)-1 di begin
;
;
;  endfor
;
;end

pro applyAdjustRv, dir, adjustedFile, options, trailer, p, nosmooth
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(adjustedFile, /read) eq 0 then begin
    print, 'Error! Unable to read ' + adjustedFile
    return
  endif

  if file_test_gz(my_filepath('rv' + options + '-binned.ad2', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/rv' + options + '-binned.ad2'
    return
  endif

  if file_test_gz(my_filepath('binnum' + options + '.ad2', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/binnum' + options + '.ad2'
    return
  endif

  if file_test_gz(my_filepath('signal' + options + '.ad2', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/signal' + options + '.ad2'
    return
  endif

  if file_test_gz(my_filepath('lambda' + options + '-binned.ad3', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/lambda' + options + '-binned.ad3'
    return
  endif

  if file_test_gz(my_filepath('cal_valid.fits', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/cal_valid.fits'
    return
  endif

  if file_test_gz(my_filepath('roi.adz', root=[dir]), /read) eq 1 then begin
    roi = adztoskyzone(readadz(dir + '/roi.adz'))
  endif

  cal_valid = readfits(my_filepath('cal_valid.fits', root=[dir]), /silent)

  rv = readad2(my_filepath('rv' + options + '-binned.ad2', root=[dir]), trailad2)
  rvmodified = readad2(adjustedFile)
  binnum = readad2(my_filepath('binnum' + options + '.ad2', root=[dir]))
  signal = readad2(my_filepath('signal' + options + '.ad2', root=[dir]))
  cubebinned = readad3(my_filepath('lambda' + options + '-binned.ad3', root=[dir]), trailer)

  temp = rvmodified

  cubeAdjusted = adjustRv(rv, rvmodified, binnum, signal, cubebinned, trailer, rvsmooth, cal_valid=cal_valid, region=roi)

  help, where(temp ne rvmodified)

  if file_test_gz(my_filepath('interfringehigh.adz', root=[dir]), /read) eq 1 then begin
    adz = adzToSkyZone(readadz(my_filepath('interfringehigh.adz', root=[dir])))
    rvmodified = resolveInterfringe(rvmodified, adz, trailer, 1.0)
    if n_elements(rvsmooth) ne 0 then rvsmooth = resolveInterfringe(rvsmooth, adz, trailer, 1.0)
  endif

  if file_test_gz(dir + 'interfringelow.adz', /read) eq 1 then begin
    adz = adzToSkyZone(readadz(my_filepath('interfringelow.adz', root=[dir])))
    rvmodified = resolveInterfringe(rvmodified, adz, trailer, -1.0)
    if n_elements(rvsmooth) ne 0 then rvsmooth = resolveInterfringe(rvsmooth, adz, trailer, -1.0)
  endif

  ; adjustRv returns the rvmodified
  ; array patched so that all pixels of a modified bin
  ; are set to the modified value
  ; We write this new map...
  toto = writead2(adjustedFile, rvmodified, trailad2)
  if n_elements(rvsmooth) ne 0 then toto = writead2(my_filepath('rv' + options + '-smooth.ad2', root=[dir]), rvsmooth, trailad2)
  toto = writead3(my_filepath('lambda' + options + '-adjusted.ad3', root=[dir]), cubeAdjusted, trailer, wise_gz=1)
  toto = writead3(my_filepath('lambda' + options + '-binned-adjusted.ad3', root=[dir]), cubeBinned, trailer, wise_gz=1)

  createRvMap, dir, 'lambda' + options + '-binned-adjusted.ad3', options + '-binned-adjusted', trailer, p, binnum='binnum' + options + '.ad2'
  createRvMap, dir, 'lambda' + options + '-adjusted.ad3', options + '-adjusted', trailer, p
end

pro createRvMap, dir, inname, options, trailer, p, binnum=binnumfile
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(my_filepath(inname, root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + inname
    return
  endif

  lambda = readad3(my_filepath(inname, root=[dir]), trailer)

  print, 'Creating RV-Mono-Continum-Width maps...'
  rv = rvMap(lambda, trailer, mono, continum, width, baryHeight, gCube)

  if n_elements(binnumfile) ne 0 then begin
    if file_test(my_filepath(binnumfile, root=[dir]), /read) eq 0 then begin
      print, 'Error! Unable to read ' + dir + '/' + binnumfile
      return
    endif

    binnum = readad2(my_filepath(binnumfile, root=[dir]))
  endif

  if file_test_gz(my_filepath('interfringehigh.adz', root=[dir]), /read) eq 1 then begin
    adz = adzToSkyZone(readadz(my_filepath('/interfringehigh.adz', root=[dir])))
    rv = resolveInterfringe(rv, adz, trailer, 1.0, binnum=binnum)
  endif

  if file_test_gz(my_filepath('interfringelow.adz', root=[dir]), /read) eq 1 then begin
    adz = adzToSkyZone(readadz(my_filepath('interfringelow.adz', root=[dir])))
    rv = resolveInterfringe(rv, adz, trailer, -1.0, binnum=binnum)
  endif

  trailad2 = {ad2_trailer}
  copyAdCoordinates, trailer, trailad2

  trailad2.thrshld = min(rv[where(rv gt -3.1e+38)])
  trailad2.step = (max(rv[where(rv gt -3.1e+38)])-min(rv[where(rv gt -3.1e+38)]))/16.0
  toto = writead2(my_filepath('rv' + options + '.ad2', root=[dir]), rv, trailad2)
  trailad2.thrshld = min(mono)
  trailad2.step = (max(mono)-min(mono))/16.0
  toto = writead2(my_filepath('mono' + options + '.ad2', root=[dir]), mono, trailad2)
  trailad2.thrshld = min(width)
  trailad2.step = (max(width)-min(width))/16.0
  toto = writead2(my_filepath('width' + options + '.ad2', root=[dir]), width, trailad2)
  trailad2.thrshld = min(continum)
  trailad2.step = (max(continum)-min(continum))/16.0
  toto = writead2(my_filepath('cont' + options + '.ad2', root=[dir]), continum, trailad2)
;  trailad2.thrshld = min(baryHeight)
;  trailad2.step = (max(baryHeight)-min(baryHeight))/16.0
;  toto = writead3(dir + '/lambda' + options + '-gaussian.ad3', gCube, trailer, data_xyz=0)
end


pro applyVoronoi, dir, inname, outname, targetSn, trailer, p, expand, voro_tryhard
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(my_filepath(inname, root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + inname
    return
  endif

  if file_test_gz(my_filepath('cal_valid.fits', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/cal_valid.fits'
    return
  endif

  cal_valid = readfits(my_filepath('cal_valid.fits', root=[dir]), /silent)

  lambda = readad3(my_filepath(inname, root=[dir]), trailer)

  sz = size(lambda)
  zsize = sz[1]
  xsize = sz[2]
  ysize = sz[3]

  if expand eq 0 then begin
    print, 'Masking lambda cube according to cal_valid'  
    lambda_temp = fltarr(sz[1], sz[2]*sz[3])
    lambda_temp[*, cal_valid] = (reform(lambda, sz[1], sz[2]*sz[3]))[*, cal_valid]
    lambda = reform(lambda_temp, sz[1], sz[2], sz[3])
  endif

  if file_test_gz(my_filepath('roi.adz', root=[dir]), /read) eq 1 then begin
    roi = adztoskyzone(readadz(my_filepath('roi.adz', root=[dir])))
    
    sz = size(lambda)

    print, 'Masking lambda cube according to roi.adz (' + strtrim(roi[0].x1,2) + ', ' + strtrim(roi[0].y1,2) + '), (' + strtrim(roi[0].x2,2) + ', ' + strtrim(roi[0].y2,2) + ')'
    lambda_temp = fltarr(sz[1], sz[2], sz[3])
    lambda_temp[*, roi[0].x1:roi[0].x2, roi[0].y1:roi[0].y2] = lambda[*, roi[0].x1:roi[0].x2, roi[0].y1:roi[0].y2]
    lambda=lambda_temp
    lambda_temp = 0
  endif

  print, 'Voronoi smoothing...'
  lambda = voronoi_3d_fast(lambda, cal_valid, targetSn, x, y, signal, noise, signal2d, noise2d, binnum, binnum2d, xnode, ynode, xbar, ybar, sn, sn2d, npixels, binA, cubeBinned, binSize, p, region=roi, tryhard=voro_tryhard)

  name = (strsplit(outname, '.', /extract))[0]

  toto = writead3(my_filepath(outname, root=[dir]), lambda, trailer, wise_gz=1)
  toto = writead3(my_filepath(name + '-binned.ad3', root=[dir]), cubeBinned, trailer, wise_gz=1)

  options = string((byte(name))[(strsplit(name, '-'))[1]:n_elements(byte(name))-1])

  toto = writead2(my_filepath('signal-' + options + '.ad2', root=[dir]), signal2d)
  toto = writead2(my_filepath('noise-' + options + '.ad2', root=[dir]), noise2d)
  toto = writead2(my_filepath('sn-' + options + '.ad2', root=[dir]), sn2d)
  toto = writead2(my_filepath('binnum-' + options + '.ad2', root=[dir]), float(binnum2d))
;  toto = writead1(dir + '/npixels-' + options + '.ad1', npixels)
;  toto = writead1(dir + '/binAccretion-' + options + '.ad1', binA)
  toto = writead2(my_filepath('binSize-' + options + '.ad2', root=[dir]), binSize)

end

pro applyOh, dir, inname, outname, trailer, sz, ohsky, ohmap_degree, sfc, p
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(my_filepath(inname, root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + inname
    return
  endif

  if file_test_gz(my_filepath('cal_valid.fits', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/cal_valid.fits'
    return
  endif

  if file_test_gz(my_filepath('roi.adz', root=[dir]), /read) eq 1 then begin
    region = adztoskyzone(readadz(dir + '/roi.adz'))
  endif

  if file_test_gz(my_filepath('sky.adz', root=[dir]), /read) eq 1 then begin
    sky = adztoskyzone(readadz(dir + '/sky.adz'))
  endif

  if file_test_gz(my_filepath('ciel.ad3', root=[dir]), /read) eq 1 then begin
    ciel_cube = readad3(my_filepath('ciel.ad3', root=[dir]))
    ciel_cube *= sfc
  endif

;  if ohsky eq 1 then begin
;    if file_test_gz(dir + '/ohmap' + sz + '.ad2', /read) eq 0 then begin
;      print, 'Error! Unable to read ' + dir + '/ohmap' + sz + '.ad2'
;      return
;    endif
;
;    ohmap = readad2(dir + '/ohmap' + sz + '.ad2')
;  endif

  cal_valid = readfits(my_filepath('cal_valid.fits', root=[dir]), /silent)

  lambda = readad3(my_filepath(inname, root=dir), trailer)
  if (ohsky eq 1) or (n_elements(ciel_cube) ne 0) then begin
    print, 'Substracting sky with ohmap'
    print, '  infile = ' + inname + ', outfile = ' + outname
    lambda = substractSky(lambda, cal_valid, ciel_cube=ciel_cube, spectrum=iciel, factors=[1.0], reg=region, skyreg=sky, /ohmap, map=ohregion, degree=ohmap_degree, plot=p)
    toto = writead3(my_filepath('iciel.ad3', root=[dir]), iciel, wise_gz=1)
    toto = writead2(my_filepath('ohmap' + sz + '.ad2', root=[dir]), ohregion)
  endif else begin
    print, 'Substracting sky with 3 passes...'
    print, '  infile = ' + inname + ', outfile = ' + outname
    lambda = substractSky(lambda, cal_valid, spectrum=iciel, factors=[0.7, 0.9, 1.0], reg=region, skyreg=sky, plot=p)
    toto = writead1(my_filepath('iciel.ad1', root=[dir]), iciel)
  endelse


  toto = writead3(my_filepath(outname, root=dir), lambda, trailer, wise_gz=1)
;  toto = writead1(dir + '/iciel.ad1', iciel)

end

pro applyFlat, dir, inname, outname, trailer, sz, cf, p, nf, ath, mxlag
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(my_filepath(inname, root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + inname
    return
  endif

  if file_test_gz(my_filepath('cal_valid.fits', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/cal_valid.fits'
    return
  endif

  if file_test_gz(my_filepath('roi.adz', root=[dir]), /read) eq 1 then begin
    roi = adztoskyzone(readadz(my_filepath('roi.adz', root=[dir])))
  endif

  if file_test_gz(my_filepath('sky.adz', root=[dir]), /read) eq 1 then begin
    skyzone = adztoskyzone(readadz(my_filepath('sky.adz', root=[dir])))
  endif

  cal_valid = readfits(my_filepath('cal_valid.fits', root=[dir]), /silent)

  lambda = readad3(dir + inname, trailer)
  size = size(lambda)
  print, 'Applying flat...'
  lambda = alignLines(lambda, offset, map, cal_valid, plot=p, flatonly=1, region=roi, continumflat=cf, ohmap=ohmap, skyzone=skyzone, alignthreshold=ath, maxlag=mxlag)

  if nf eq 0 then begin
    toto = writead3(dir + outname, lambda, trailer, wise_gz=1)
  endif
;  toto = writead2(dir + '/offset.ad2', offset)
  toto = writead2(my_filepath('skyflat.ad2', root=[dir]), map)
;  toto = writead2(dir + '/ohmap' + sz + '.ad2', ohmap)

  if p eq 1 then begin    
    loadct, 40, /silent
    window, 1, xsize=size[2], ysize=size[3]
    tvscl, offset
  endif

end

pro applyAlign, dir, inname, outname, trailer, pa, sz, cf, p, nf, ath, mxlag
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(my_filepath(inname, root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + inname
    return
  endif

  if file_test_gz(my_filepath('cal_valid.fits', root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + '/cal_valid.fits'
    return
  endif

  if file_test_gz(my_filepath('roi.adz', root=[dir]), /read) eq 1 then begin
    roi = adztoskyzone(readadz(dir + '/roi.adz'))
  endif

  if file_test_gz(my_filepath('sky.adz', root=[dir]), /read) eq 1 then begin
    skyzone = adztoskyzone(readadz(dir + '/sky.adz'))
  endif

  cal_valid = readfits(my_filepath('cal_valid.fits', root=[dir]), /silent)

  lambda = readad3(dir + inname, trailer)
  size = size(lambda)
  print, 'Aligning lines and applying flat...'
  lambda = alignLines(lambda, offset, map, cal_valid, plot=p, npass=pa, region=roi, continumflat=cf, ohmap=ohmap, noflat=nf, skyzone=skyzone, alignthreshold=ath, maxlag=mxlag)

  toto = writead3(my_filepath(outname, root=[dir]), lambda, trailer, wise_gz=1)
  toto = writead2(my_filepath('offset.ad2', root=[dir]), offset)
  toto = writead2(my_filepath('skyflat.ad2', root=[dir]), map)
;  toto = writead2(dir + '/ohmap' + sz + '.ad2', ohmap)

  if p eq 1 then begin    
    loadct, 40, /silent
    window, 1, xsize=size[2], ysize=size[3]
    tvscl, offset
  endif

end

pro applySmooth, dir, inname, outname, sl, trailer, p
COMPILE_OPT IDL2, HIDDEN

  if file_test_gz(my_filepath(inname, root=[dir]), /read) eq 0 then begin
    print, 'Error! Unable to read ' + dir + inname
    return
  endif

  lambda = readad3(my_filepath(inname, root=[dir]), trailer)
  print, 'Applying spectral smoothing of type ' + strtrim(sl, 2)
  lambda = spectralSmooth(lambda, sl)

  toto = writead3(my_filepath(outname, root=[dir]), lambda, trailer, wise_gz=1)

end

pro computelambda, dir, outname, cieldir, adadir, correctGuiding, cgnb, trailer, sfn, p, ignorecycle=ignorecycle, create_neb=cn, correct_continuum=correct_continuum
COMPILE_OPT IDL2, HIDDEN

  if n_elements(cn) eq 0 then cn = 0

  if (file_test_gz(my_filepath('cal_prb.ad2', root=[dir]), /read) eq 0) then begin
    print, 'Error! Unable to read ' + cieldir + '/' + adadir + '/cal_prb.ad2'
    return
  endif

  prb = readad2(my_filepath('cal_prb.ad2', root=[dir]))

  cal_cen = readad1(my_filepath('cal_cen.ad1', root=[dir]))

  if sfn eq 1 then begin
    if file_test_gz(my_filepath('neb.ad3', root=[dir]), /read) eq 1 then begin
      print, 'Using neb file...'
      neb = readad3(my_filepath('neb.ad3', root=[dir]), trailer)
      lambda = createLambda(neb, prb)

      toto = writead3(my_filepath(outname, root=[dir]), lambda, trailer, wise_gz=1)
      return
    endif else begin
      print, 'Error! Unable to read ' + dir + '/neb.ad3!'
      return
    endelse
  endif
  
  if correctGuiding eq 1 then begin
    if file_test_gz(my_filepath('stars.ad1', root=[dir]), /read) eq 1 then begin
      print, 'Using already defined stars for guiding correction...'
      stars = readad1(my_filepath('stars.ad1', root=[dir]))
      xstar = stars[0:n_elements(stars)/2-1]
      ystar = stars[n_elements(stars)/2:n_elements(stars)-1]
    endif

    print, 'Integrating ada files from ' + my_filepath('', subdirectory=[adadir], root=cieldir) + ' and correcting guiding...'
    lambda = integrateAdaAndCorrectGuiding(my_filepath('', subdirectory=[adadir], root=cieldir), prb, cal_cen, cgnb, xstar, ystar, flux, p, ignorecycle=ignorecycle, correct_continuum=correct_continuum, stars_file=my_filepath('stars.ad1', root=[dir]))

    neb = createNeb(lambda, prb)

    toto = writead1(my_filepath('flux.ad1', root=[dir]), flux)
    toto = writead1(my_filepath('stars.ad1', root=[dir]), [xstar, ystar])
  endif else begin
    print, 'Integrating ada files from ' + cieldir + '/' + adadir + '...'
    neb = integrateAda(my_filepath('', root=cieldir, subdirectory=[adadir]), ignorecycle=ignorecycle)
    print, 'Creating lambda map...'
    lambda = createLambda(neb, prb)
  endelse

  if cn eq 1 then toto = writead3(my_filepath('neb.ad3', root=[dir]), neb, trailer, wise_gz=1)
  toto = writead3(my_filepath(outname, root=[dir]), lambda, trailer, wise_gz=1)

end


pro applyCalibration, dir, cieldir, calibdir, calibwl, trailer, caladh, p
COMPILE_OPT IDL2, HIDDEN

  calibdir = strupcase(calibdir)

  if caladh eq 1 then begin
    print, 'Warning! Using ADHOC phase... I hope you know what you are doing!'
    bru = readad2((file_search(my_filepath('[pP][hH][aA][sS]_[bB][rR][uU].[aA][dD]2', root=[dir])))[0], trailer)
    toto = writead2(my_filepath('cal_bru.ad2', root=[dir]), bru, trailer)
    data = readad2((file_search(my_filepath('[pP][hH][aA][sS]_[pP][rR][bB].[aA][dD]2', root=[dir])))[0], trailer)
    toto = writead2(my_filepath('cal_prb.ad2', root=[dir]), data, trailer)
    data = readad2((file_search(my_filepath('[rR][vV]_[cC][aA][lL].[aA][dD]2', root=[dir])))[0], trailer)
    toto = writead2(my_filepath('cal_rv.ad2', root=[dir]), data, trailer)
    sum = readad2((file_search(my_filepath('[sS][uU][mM]_[cC][aA][lL].[aA][dD]2', root=[dir])))[0], trailer)
    toto = writead2(my_filepath('cal_sum.ad2', root=[dir]), sum, trailer)

    cen = findCalibrationCenter(bru)
    toto = writead1(my_filepath('cal_cen.ad1', root=[dir]), cen)

    sz = size(bru)
    dist_circle, cir, sz[1], cen[0], cen[1]
    firstInterfringePosition = 100
    med = median(sum[where(cir lt firstInterfrangePosition)])
    validCalib = where(sum ge med/2, complement=nogood)
    writefits, my_filepath('cal_valid.fits', root=[dir]), validCalib

    return
  endif

  if file_test_gz(my_filepath('calibration.ad3', root=[dir]), /read) eq 0 then begin
    for i=0, n_elements(calibdir)-1 do begin
      file = file_search(my_filepath(calibdir[i] + '.[aA][dD]3', root=cieldir, subdirectory=[calibdir[i]]))
      if file_test_gz(file[0], /read) eq 1 then begin
        print, 'Reading calibration file ' + file[0]
        if n_elements(calibration) eq 0 then begin
          calibration = readad3(file[0])
          normal = total(calibration)
        endif else begin
          calib2 = readad3(file[0])
          normal2 = total(calib2)
          print, '  Normalisation is ' + strtrim(normal/normal2,2)
          calibration = calibration + calib2*(normal/normal2)
        endelse
      endif else begin
        print, 'Warning! Unable to find a calibration file in ' + cieldir + '/' + calibdir[i]
      endelse
    endfor

    calib2=0

    if n_elements(calibration) eq 0 then begin
      print, 'Error! No calibration files found!'
      return
    endif

    toto = writead3(my_filepath('calibration.ad3', root=[dir]), calibration, trailer, wise_gz=1)
  endif else begin
    calibration = readad3(my_filepath('calibration.ad3', root=[dir]))
  endelse


  print, 'Calibrating...'
  calibrate, calibration, trailer, calibwl, bru, prb, sum, rvcal, finesse, validCalib, center=cen

  print, 'Writing calibration files...'
  toto = writead2(my_filepath('cal_bru.ad2', root=[dir]), bru)
  toto = writead2(my_filepath('cal_prb.ad2', root=[dir]), prb)
  toto = writead2(my_filepath('cal_sum.ad2', root=[dir]), sum) 
  toto = writead2(my_filepath('cal_rv.ad2', root=[dir]), rvcal)
  toto = writead1(my_filepath('cal_cen.ad1', root=[dir]), cen)
  writefits, my_filepath('cal_valid.fits', root=[dir]), validCalib

  if p eq 1 then begin
    sz = size(prb)
    window, 0, xsize=sz[1], ysize=sz[2]
    loadct, 40, /silent
    tvscl, prb
    tvcircle, 19, cen[0], cen[1], 250
    tvbox, 9, cen[0], cen[1], 250
    tvbox, 1, cen[0], cen[1], 250
  endif

;  print, 'Computing calibration finesse...'
;  finesse = computecalibrationfinesse(calibration, prb, cen, plot=p)
  toto = writead1(my_filepath('cal_finesse.ad1', root=[dir]), finesse)


end

function is_smaller_mtime, unit, units
COMPILE_OPT IDL2, HIDDEN
  small = 0
  for i=0, n_elements(units)-1 do if unit.mtime le units[i].mtime then small=1
  return, small
end


pro computeeverything, dir, cieldir, calibdir, adadir, targetsn, align=a, thresholdalign=ath, maxalign=maxalign, passalign=pa, remove_sky=sky, $
ohmap_remove_sky=ohsky, ohmap_degree=ohmap_degree, spectral_smooth=sl, voro=v, plot=p, correctguiding=correctGuiding, guidingnobreak=cgnb, coadd=co, $
continumflat=cf, noflat=nf, autorvmonolevel=autorvmonolevel, ignorecycle=ignorecycle, expand=expand, adjustrv_nosmooth=adjustrv_nosmooth, $
tryhard_voro=voro_tryhard, calibadhoc=caladh, startfromneb=sfn, adacieldir=adacieldir, scalefactorciel=sfc, correct_continuum=correct_continuum
COMPILE_OPT IDL2

;  cd, '.', current=toto
;  !dir = toto

  if n_elements(a) eq 0 then a=0
  if n_elements(sky) eq 0 then sky=1
  if n_elements(v) eq 0 then v=1
  if n_elements(p) eq 0 then p=0
  if n_elements(sl) eq 0 then sl=1
  if n_elements(correctGuiding) eq 0 then correctGuiding=0
  if n_elements(cgnb) eq 0 then cgnb=1
  if n_elements(co) eq 0 then co=0
  if n_elements(cf) eq 0 then cf=0
  if n_elements(ohsky) eq 0 then ohsky=1
  if n_elements(nf) eq 0 then nf=1
  if n_elements(autorvmonolevel) eq 0 then autorvmonolevel=2.0
  if n_elements(expand) eq 0 then expand=0
  if n_elements(adjustrv_nosmooth) eq 0 then adjustrv_nosmooth=0
  if n_elements(voro_tryhard) eq 0 then voro_tryhard = 0
  if n_elements(caladh) eq 0 then caladh = 0
  if n_elements(sfn) eq 0 then sfn = 0
  if n_elements(sfc) eq 0 then sfc = 1.0
  if n_elements(correct_continuum) eq 0 then correct_continuum=0

  if (nf eq 0) or (a eq 1) then begin
    print, '**** WARNING ****'
    print, ' Alignment on the sky emission lines and using them for flat-fielding is innacurate due to interference filter transmission inhomogenities.'
    print, ' You are warned...'
  endif

  if (correct_continuum eq 1) then begin
    print, '**** WARNING ****'
    print, 'Continuum correction in the ADA is still experimental.
    print, 'You are warned...'
  endif



  gzfiles = file_search(my_filepath('*.ad3.gz', root=[dir]))
  if gzfiles[0] ne '' then begin
    for i=0, n_elements(gzfiles)-1 do begin
      spawn, 'gzip -lv ' + gzfiles[i], out
      out = out[1]
      out = strsplit(out, ' ', /extract)
;      info = file_info(
      print, 'Touching ' + gzfiles[i]
      spawn, 'touch --date="' + out[2] + ' ' + out[3] + ' ' + out[4] + '" ' + gzfiles[i]
    endfor
  endif

  loadct, 0, /silent

  ; On a besoin de fichiers crees par l'alignement ou le flat pour ohsky...
;  if (nf eq 1) and (a eq 0) then ohsky=0

;  if n_elements(keep) eq 0 then k=1

  if dir eq '' then dir = '.'

;  if p eq 1 then begin
;    ys = (correctGuiding + (sl eq 0)?0:1 + a + sky + v) * 10
;    window, 29, xsize=200, ysize=50, xpos=700, ypos=487, title='Traitement de ' + dir
;    loadct, 13, /silent
;    y = ys - 10
;    if correctGuiding eq 1 then begin
;      xyouts, 10, y, '- Correction du guidage'
;      y = y - 10
;    endif
;    if sl ne 0 then begin
;      xyouts, 10, y, '- Lissage en Z (' + strtrim(sl, 2) + ')'
;      y = y - 10
;    endif
;    if a eq 1 then begin
;      xyouts, 10, y, '- Alignement'
;      y = y - 10
;    endif
;    if sky eq 1 then begin
;      if ohsky eq 0 then begin
;        xyouts, 10, y, '- Soustraction OH'
;        y = y - 10
;      endif else begin
;        xyouts, 10, y, '- Soustraction OHmap'
;        y = y - 10
;      endelse
;    endif
;    if v eq 1 then begin
;      xyouts, 10, y, '- Lissage Voronoi, sn=' + strtrim(targetSn, 2)
;      y = y - 10
;    endif
;;    XYOUTS, 10, 40, 
;;    XYOUTS, 10, 10, targetSN, /DEVICE
;  endif

  print, 'Entering ' + dir

  adp = file_search(my_filepath('*.[aA][dD][pP]', root=[dir]))

  if adp eq '' then begin
    print, 'Error! Unable to find an adp file!'
    return
  endif

  trailer = adpToAd3Trailer(adp, calibwl)

  align = a eq 1?'-aligned':''
  if ohsky eq 1 then begin
    oh    = sky eq 1?'-OHmap':''
  endif else begin
    oh    = sky eq 1?'-OH':''
  endelse
  sz    = sl ge 1?'-SZ'+strtrim(long(sl),2):''
  voro  = v eq 1?'-sn'+add_zeros(strtrim(long(targetsn),2),2):''
  coadd = co eq 1?'-coadd':''

  print, '  Align: ' + align
  print, '  Oh: ' + oh
  print, '  SZ: ' + sz
  print, '  Voro: ' + voro
  print, '  Coadd: ' + coadd

  info_adp = file_info_gz(adp)
  info_neb = file_info_gz(my_filepath('neb.ad3', root=[dir]))
  info_cal = file_info_gz(my_filepath('cal_prb.ad2', root=[dir]))
  info_lambda = file_info_gz(my_filepath('lambda.ad3', root=[dir]))
  info_sz = file_info_gz(my_filepath('lambda' + sz + '.ad3', root=[dir]))
  info_align = file_info_gz(my_filepath('lambda' + sz + align + '.ad3', root=[dir]))
  info_roi = file_info_gz(my_filepath('roi.adz', root=[dir]))
  info_sky = file_info_gz(my_filepath('sky.adz', root=[dir]))
  info_oh = file_info_gz(my_filepath('lambda' + sz + align + oh + coadd + '.ad3', root=[dir]))
  info_voro = file_info_gz(my_filepath('lambda' + sz + align + oh + coadd + voro + '.ad3', root=[dir]))
  info_voro2 = file_info_gz(my_filepath('lambda' + sz + align + oh + coadd + voro + '-binned.ad3', root=[dir]))    
  info_rv = file_info_gz(my_filepath('rv' + sz + align + oh + coadd + voro + '.ad2', root=[dir]))
  info_rv_novoro = file_info_gz(my_filepath('rv' + sz + align + oh + coadd + '.ad2', root=[dir]))

  if v eq 1 then begin
    info_rv2 = file_info_gz(my_filepath('rv' + sz + align + oh + coadd + voro + '-binned.ad2', root=[dir]))
  endif else begin
    info_rv2 = info_rv
  endelse

  adjustedFile = file_search(my_filepath('rv' + sz + align + oh + coadd + voro + '-binned-adjusted?.ad2', root=[dir]))
;  print, adjustedFile

  if adjustedFile[0] ne '' then begin
    adjustedFile = adjustedFile[n_elements(adjustedFile)-1]
    info_rv_adjusted = file_info_gz(adjustedFile)
  endif else begin
    ; On veut un file_info d'un fichier inexistant...
    adjustedFile = my_filepath('rv' + sz + align + oh + coadd + voro + '-binned-adjusted.ad2', root=[dir])
    info_rv_adjusted = file_info_gz(adjustedFile)
  endelse

  info_rv_adjusted_final = file_info_gz(my_filepath('rv' + sz + align + oh + coadd + voro + '-adjusted.ad2', root=[dir]))

  info_interfringehigh = file_info_gz(my_filepath('interfringehigh.adz', root=[dir]))
  info_interfringelow = file_info_gz(my_filepath('interfringelow.adz', root=[dir]))
;  info_ciel = file_info_gz(dir + '/ciel.adz')

  rebuild_rv_novoro = is_smaller_mtime(info_rv_novoro, [info_adp, info_oh, info_roi, info_align, info_sz, info_lambda, info_cal, info_interfringelow, info_interfringehigh])

  rebuild_rv     = is_smaller_mtime(info_rv, [info_adp, info_voro, info_oh, info_roi, info_align, info_sz, info_lambda, info_cal, info_interfringelow, info_interfringehigh])
  rebuild_rv = rebuild_rv + is_smaller_mtime(info_rv2, [info_adp, info_voro, info_oh, info_roi, info_align, info_sz, info_lambda, info_cal, info_interfringelow, info_interfringehigh])

  if info_rv_adjusted.mtime ne 0 then begin
    rebuild_rv_adjusted = is_smaller_mtime(info_rv_adjusted_final, [info_adp, info_rv_adjusted, info_roi, info_interfringelow, info_interfringehigh])
    print, '  Adjusted rv file found... (' + adjustedFile + ')'
  endif else begin
    rebuild_rv_adjusted = 0
  endelse

  rebuild_voro   = is_smaller_mtime(info_voro, [info_adp, info_oh, info_roi, info_sky, info_align, info_sz, info_lambda, info_cal])
  rebuild_voro2  = rebuild_voro + is_smaller_mtime(info_voro2, [info_adp, info_oh, info_roi, info_sky, info_align, info_sz, info_lambda, info_cal])
  rebuild_oh     = is_smaller_mtime(info_oh, [info_adp, info_align, info_roi, info_sky, info_sz, info_lambda, info_cal])
  rebuild_align  = is_smaller_mtime(info_align, [info_adp, info_roi, info_sz, info_lambda, info_cal])
  rebuild_sz     = is_smaller_mtime(info_sz, [info_adp, info_lambda, info_cal])
  if sfn eq 1 then begin
    rebuild_lambda = is_smaller_mtime(info_lambda, [info_adp, info_cal, info_neb])
  endif else begin
    rebuild_lambda = is_smaller_mtime(info_lambda, [info_adp, info_cal])
  endelse

;  if (info_interfringehigh.mtime + info_interfringelow.mtime) ne 0 then begin
;    rebuild_interfringe = is_smaller_mtime(info_interfringehigh, [info_rv, info_rv2])
;    rebuild_interfringe = rebuild_interfringe + is_smaller_mtime(info_interfringelow, [info_rv, info_rv2])
;  endif else begin
    rebuild_interfringe = 0
;  print, rebuild_rv, rebuild_voro, rebuild_oh, rebuild_align, rebuild_sz, rebuild_lambda

  if (rebuild_rv ne 0) or (rebuild_rv_novoro ne 0) then begin
    if rebuild_voro ne 0 then begin
      if rebuild_oh ne 0 then begin
        if rebuild_align ne 0 then begin
          if rebuild_sz ne 0 then begin
            if rebuild_lambda ne 0 then begin
              if file_test_gz(my_filepath('cal_prb.ad2', root=[dir]), /read) eq 0 then begin
                ; CALIBRATION
                applyCalibration, dir, cieldir, calibdir, calibwl, trailer, caladh, p
              endif

              ; CREATION DU LAMBDA           
              computeLambda, dir, 'lambda.ad3', cieldir, adadir, correctGuiding, cgnb, trailer, sfn, p, ignorecycle=ignorecycle, create_neb=1, correct_continuum=correct_continuum
              if n_elements(adacieldir) ne 0 then begin
                computeLambda, dir, 'ciel.ad3', cieldir, adacieldir, 0, 0, trailer, 0, p, ignorecycle=ignorecycle, create_neb=0
              endif

            endif

            ; SPECTRAL SMOOTHING
            if sl ge 1 then begin
              applySmooth, dir, 'lambda.ad3', 'lambda' + sz + '.ad3', sl, trailer, p
            endif
          endif

          ; ALIGNEMENT SUR LES RAIES DU CIEL
          if a eq 1 then begin
            applyAlign, dir, 'lambda' + sz + '.ad3', 'lambda' + sz + align + '.ad3', trailer, pa, sz, cf, p, nf, ath, maxalign
          endif else begin
            if nf ne 1 then begin
              applyFlat, dir, 'lambda' + sz + '.ad3', 'lambda' + sz + '.ad3', trailer, sz, cf, p, nf, ath, maxalign
            endif
          endelse

        endif

        ; SOUSTRACTION DU OH
        if sky eq 1 then begin
          applyOh, dir, 'lambda' + sz + align + '.ad3', 'lambda' + sz + align + oh + '.ad3', trailer, sz, ohsky, ohmap_degree, sfc, p
        endif
      endif

      ; VORONOI SMOOTHING
      if v eq 1 then begin
        applyVoronoi, dir, 'lambda' + sz + align + oh + coadd + '.ad3', 'lambda' + sz + align + oh + coadd + voro + '.ad3', targetSn, trailer, p, expand, voro_tryhard
      endif
    endif

    ; CREATION DES CARTES 2D
    if rebuild_rv ne 0 then begin
      createRvMap, dir, 'lambda' + sz + align + oh + coadd + voro + '.ad3', sz + align + oh + coadd + voro, trailer, p
      if v eq 1 then begin
        createRvMap, dir, 'lambda' + sz + align + oh + coadd + voro + '-binned.ad3', sz + align + oh + coadd + voro + '-binned', trailer, p, binnum='binnum' + sz + align + oh + coadd + voro + '.ad2'
  
        rv = readad2(my_filepath('rv' + sz + align + oh + coadd + voro + '-binned.ad2', root=[dir]))
        mono = readad2(my_filepath('mono' + sz + align + oh + coadd + voro + '-binned.ad2', root=[dir]))

        rva = automaticallyAdjustRv(rv, mono, autorvmonolevel, trailer.xik)

        trailad2 = {ad2_trailer}
        copyAdCoordinates, trailer, trailad2
        trailad2.thrshld = min(rva)
        trailad2.step = (max(rva)-min(rva))/16.0
        toto = writead2(my_filepath('rv' + sz + align + oh + coadd + voro + '-binned-adjusted.ad2', root=[dir]), rva)

        applyAdjustRv, dir, my_filepath('rv' + sz + align + oh + coadd + voro + '-binned-adjusted.ad2', root=[dir]), sz + align + oh + coadd + voro, trailer, p
      endif
    endif
 
    if rebuild_rv_novoro ne 0 then begin
      createRvMap, dir, 'lambda' + sz + align + oh + coadd + '.ad3', sz + align + oh + coadd, trailer, p
    endif

  endif

  if rebuild_rv_adjusted ne 0 then begin
    applyAdjustRv, dir, adjustedFile, sz + align + oh + coadd + voro, trailer, p, adjustrv_nosmooth
  endif

;  if rebuilb_interfringe ne 0 then begin
;    applyInterfringe, dir, sz + align + oh + coadd + voro, trailer, p
;  endif


end
