function integrate_fits_and_correct_guiding, filesin, prb, reffiles=reffiles

  ; reffiles sont les fichiers
  ; de reference ou la position des etoiles sera prise.

  if n_elements(reffiles) eq 0 then reffiles = filesin

  if n_elements(reffiles) ne n_elements(filesin) then begin
    print, 'filesin and reffiles must have the same number of elements'
    return, -1
  endif

  im = readfits(reffiles[0])
  sz = size(im)

  zsize = n_elements(filesin)
  lambda = fltarr(zsize, sz[1], sz[2])

  window, 0, xsize=sz[1]/2, ysize=sz[2]/2
  tvscl, rescale(rebin(im, sz[1]/2, sz[2]/2), 0.01, 0.99)
  
  xstar = 1.0
  ystar = 1.0

  print, 'Click on reference stars...'
  print, 'Double-click to add no more.'

  while 1 do begin
    cursor, x, y, /device, /up
    cntrd, im, x*2, y*2, xc, yc, 20

    if xc eq -1 then begin
      print, 'Unable to find reference star at (' + strtrim(x*2,2), + ', ' + strtrim(y*2,2) + ')'
      continue
    endif

    if (round(xc) eq round(xstar[0])) and (round(yc) eq round(ystar[0])) then break

    print, 'Found star at (' + strtrim(xc,2), + ', ' + strtrim(yc,2) + ')'

    loadct, 39, /silent
    tvcircle, 5, xc/2, yc/2, color=240
    loadct, 0, /silent

    xstar = [xc, xstar]
    ystar = [yc, ystar]
  endwhile

  xstar = xstar[0:n_elements(xstar)-2]
  ystar = ystar[0:n_elements(ystar)-2]

  print, 'Found ' + strtrim(n_elements(xstar),2) + ' stars'

  for i=0, n_elements(filesin)-1 do begin
    print, 'Reading file ' + filesin[i]
    im = readfits(filesin[i])
    refim = readfits(reffiles[i])

    tvscl, rescale(rebin(im, sz[1]/2, sz[2]/2), 0.01, 0.99)

    if n_elements(old_xstar) eq 0 then begin
      old_xstar = xstar
      old_ystar = ystar
    endif else begin
      old_xstar = xxstar
      old_ystar = yystar

      bad = where(xxstar eq -1)
      if bad[0] ne -1 then begin
        old_xstar[bad] = xstar[bad]
        old_ystar[bad] = ystar[bad]
      endif
    endelse

    xxstar = 0
    yystar = 0

    for j=0, n_elements(xstar)-1 do begin
      cntrd, refim, old_xstar[j], old_ystar[j], xc, yc, 20

      xxstar = [xxstar, xc]
      yystar = [yystar, yc]

      if xc eq -1 then begin
        print, 'Unable to find reference star that was at (' + strtrim(xstar[j],2), + ', ' + strtrim(ystar[j],2) + ').'
        print, '  Marking it as bad.'
      endif else begin
        loadct, 39, /silent
        tvcircle, 5, xc/2, yc/2, color=240
        loadct, 0, /silent
      endelse
    endfor

    xxstar = xxstar[1:n_elements(xxstar)-1]
    yystar = yystar[1:n_elements(yystar)-1]

    good = where(xxstar ne -1)
    meanxref = mean(xstar[good])
    meanyref = mean(ystar[good])
    meanx = mean(xxstar[good])
    meany = mean(yystar[good])

    dx = meanx - meanxref
    dy = meany - meanyref

    print, 'Offset for file ' + strtrim(i,2) + ' is (' + strtrim(dx,2) + ', ' + strtrim(dy,2) + ')'

    tempprb = interpolate(prb, findgen(sz[1])+dx, findgen(sz[2])+dy, /grid, cubic=-0.6)
    im = interpolate(im, findgen(sz[1])+dx, findgen(sz[2])+dy, /grid, cubic=-0.6)

    rt = fltarr(zsize)
    tmp = findgen(zsize*3)
    for x=0, sz[1]-1 do begin
      for y=0, sz[2]-1 do begin
        rt[i] = im[x,y]
        t = [rt, rt, rt]

        lambda[*,x,y] += (interpolate([t, t, t], tmp+(tempprb[x,y] mod zsize)))[zsize:zsize*2-1]
      endfor
    endfor
  endfor

  return, lambda


end
