function remove_zero_space, string
COMPILE_OPT IDL2,HIDDEN

  temp = byte(string)

  while temp[0] eq 32 do temp=temp[1:n_elements(temp)-1]
  while temp[n_elements(temp)-1] eq 32 do temp=temp[0:n_elements(temp)-2]

  if strmatch(string, '*.*') ne 0 then begin
    while temp[0] eq byte('0') do temp=temp[1:n_elements(temp)-1]
    while temp[n_elements(temp)-1] eq byte('0') do temp=temp[0:n_elements(temp)-2]

    while temp[n_elements(temp)-1] eq byte('.') do temp=temp[0:n_elements(temp)-2]
  endif

  return, string(temp)

end

pro myplot, x, y, out=o, f=filename, mypsym=mpsym, plot_mean=pma, plot_bary_pos=pbp, plot_median=pme, plot_stddev=pstd, plot_absdev=pabs, print=print, _extra=e

  if o eq 'ps' then begin
    set_plot, 'ps'
    device, /color, bits=8, /encapsulated, file=filename, preview=1, pre_depth=1, pre_xsize=8, pre_ysize=6, xsize=16, ysize=12 ;, set_font='-urw-urw bookman l-medium-r-normal--20-0-0-0-p-0-adobe-standard'
  endif

  xrange = max(x)-min(x)
  yrange = max(y)-min(y)
  xpos = min(x)+xrange*0.05
  ypos = min(y)+yrange*0.9

  if o ne 'ps' then begin
    ystep = yrange*0.025
  endif else begin
    ystep = yrange*0.05
  endelse

  plot, x, y, psym=mpsym, _extra=e ;xrange=[min,max], xstyle=1, yrange=[0,mmax], ystyle=1, background=255, color=0, title='SN distribution before adaptive binning', xtitle='SN', /xlog, psym=10

  if keyword_set(pma) then begin
    me = mean(y)
    if keyword_set(print) then begin
      xyouts, xpos, ypos, 'mean: ' + remove_zero_space(strtrim(float(long(me*100))/100.0,2)), /data, _extra=e
      ypos -= ystep
    endif    
    oplot, minmax(x), [me, me], linestyle=0, psym=0, _extra=e
  endif

  if keyword_set(pme) then begin
    me = median(y)
    if keyword_set(print) then begin
      xyouts, xpos, ypos, 'median: ' + remove_zero_space(strtrim(float(long(me*100))/100.0,2)), /data, _extra=e
      ypos -= ystep
    endif   
    oplot, minmax(x), [me, me], linestyle=0, psym=0, _extra=e
  endif

  if keyword_set(pmep) then begin
    me = median(y, pos)
    oplot, [x[pos], x[pos]], minmax(y), linestyle=2, psym=0, _extra=e
  endif

  if keyword_set(pbp) then begin
    pos = total(x*y)/total(y)
    print, pos
    print, min(x)
    oplot, [pos, pos], minmax(y), linestyle=2, psym=0, _extra=e
  endif

  if keyword_set(pstd) then begin
    std = stddev(y)
    if keyword_set(print) then begin
      xyouts, xpos, ypos, 'stddev: ' + remove_zero_space(strtrim(float(long(std/5.0*100.0*100.0))/100.0,2)) + ' %', /data, _extra=e
      ypos -= ystep
    endif    
    me = mean(y)
    oplot, minmax(x), [me+std, me+std], linestyle=2, psym=0, _extra=e
    oplot, minmax(x), [me-std, me-std], linestyle=2, psym=0, _extra=e
  endif
  if keyword_set(pabs) then begin
    std = meanabsdev(y)
    if keyword_set(print) then begin
      xyouts, xpos, ypos, 'meanabsdev: ' + remove_zero_space(strtrim(float(long(std/5.0*100*100))/100.0,2)) + ' %', /data, _extra=e
      ypos -= ystep
    endif
    me = mean(y)
    oplot, minmax(x), [me+std, me+std], linestyle=2, psym=0, _extra=e
    oplot, minmax(x), [me-std, me-std], linestyle=2, psym=0, _extra=e
  endif

  if o eq 'png' then begin
    toto = tvrd(true=1)
    write_png, filename, toto
  endif

  if o eq 'ps' then device, /close

end
