pro weightedCentroid, x, y, density, xbar, ybar
COMPILE_OPT IDL2, HIDDEN
  d = density

  mass = total(d)

  if mass eq 0 then begin
    mass = n_elements(d)
    d[*] = 1.0
  endif

  xBar = total(x*d)/mass
  yBar = total(y*d)/mass

end

function triangulateBinnedCube, cube, binnum
COMPILE_OPT IDL2, HIDDEN

  triangulate, xNode, yNode, triangles, b
  
  xmin = floor(min(xNode))>0
  xmax = ceil(max(xNode))<(sz[2]-1)
  ymin = floor(min(yNode))>0
  ymax = ceil(max(yNode))<(sz[3]-1)

  for i=0, sz[1]-1 do begin
    cubeSmooth[i, xmin:xmax, ymin:ymax] = trigrid(xnode, ynode, outraies[i, *], triangles, nx=xmax-xmin+1, ny=ymax-ymin+1)
  endfor

  return, -1

end




function smoothBinnedCube, cube, binnum, cal_valid=cal_valid, region=reg, triangulate=tr, withrv=wrv
COMPILE_OPT IDL2

  if n_elements(tr) ne 0 then begin
    if tr[0] eq 1 then return, triangulateBinnedCube(cube, binnum)
  endif

  if n_elements(wrv) eq 0 then wrv=0

  sz = size(cube)
  xsize = sz[2]
  ysize = sz[3]
  zsize = sz[1]
  cubeFinal = cube

  nbins = max(binnum)

  if n_elements(reg) ne 0 then begin
    tosmooth = bytarr(sz[2], sz[3])
    for i=0, n_elements(reg)-1 do begin
      tosmooth[reg[i].x1:reg[i].x2, reg[i].y1:reg[i].y2] = 1
    endfor
    tosmooth = where(tosmooth eq 1)
  endif

  if n_elements(cal_valid) ne 0 then begin
    if n_elements(tosmooth) eq 0 then tosmooth = bytarr(sz[2], sz[3])+1
    
    tosmooth2 = bytarr(sz[2], sz[3])
    tosmooth2[cal_valid] = 1

    tosmooth = tosmooth and tosmooth2
  endif

  for i=0, nbins do begin
    pix = where(binnum eq i)
    if pix[0] eq -1 then begin
      print, 'Unable to find pixels for bin ' + strtrim(i,2)
      continue
    endif

    if (n_elements(reg) ne 0) or (n_elements(cal_valid) ne 0) then begin
      pixtemp = -1
      n = n_elements(pix)
      for j=0, n-1 do begin
        idxtemp = where(tosmooth eq pix[j])
        if idxtemp[0] ne -1 then pixtemp = [pixtemp, pix[j]]
      endfor
      if n_elements(pixtemp) eq 1 then continue
      pix = pixtemp[1:n_elements(pixtemp)-1]
    endif

    n = n_elements(pix)
    pix = array_indices(binnum, pix)

    rad = sqrt(round(float(n)/!pi))
    rad = rad>1
    rad = round(rad)

;    print, float(i)/nbins*100, n, rad

    if n ne 1 then begin    
      dist_circle, gauss, rad*2
      gauss = exp(-((gauss/(rad/2.0))^2)/2)    

    ; On veut que l'integrale de la gaussienne donne 1
      gauss = gauss / total(gauss)
    endif

;    tvscl, congrid(gauss, 512, 512)
;    print, 'Gauss: ', total(gauss), n
;    print, minmax(gauss)

    for j=0, n-1 do begin
      x = pix[0,j]
      y = pix[1,j]

      xmin = (x-rad)>0
      xmax = (x+rad)<(xsize)
      ymin = (y-rad)>0
      ymax = (y+rad)<(ysize)

;      if total(cube[*,x,y]*cube[*,x,y]) eq 0 then continue

      if rad gt 1 then begin
        gaussr = gauss[xmin-x+rad:xmax-x+rad-1, ymin-y+rad:ymax-y+rad-1]

        gaussr = gaussr * ((rad*2)^2/float(n_elements(gaussr)))

        region = cube[*, xmin:xmax-1, ymin:ymax-1]
;        region = cube[*, xmin-x+rad:xmax-x+rad-1, ymin-y+rad:ymax-y+rad-1]

        if wrv eq 0 then zlimit = zsize-1 else zlimit = zsize-2

        for k=0, zlimit do begin
          cubeFinal[k, x, y] = total(reform(region[k,*,*]) * gaussr)
        endfor

        if wrv ne 0 then begin
          ; Un cas special pour le dernier
          ; channel. C'est en fait la carte RV

          n2 = where((region[k,*,*] le -3.1e+38) and (gaussr gt 0))
          if n2[0] eq -1 then begin
            cubeFinal[zsize-1, x, y] = total(reform(region[zsize-1,*,*]) * gaussr)
          endif else begin
            toadd = total(gaussr[n2])
            gaussr[n2] = 0
            n2 = where((region[k,*,*] ge -3.1e+38) and (gaussr gt 0))
            if n2[0] eq -1 then begin
              cubeFinal[zsize-1, x, y] = -3.1e+38
            endif else begin
              toadd = toadd / float(n_elements(n2))
              gaussr[n2] = gaussr[n2] + toadd
              cubeFinal[zsize-1, x, y] = total(reform(region[zsize-1,*,*]) * gaussr)
            endelse
          endelse
      endif

      endif else begin
        cubeFinal[*,x,y] = cube[*,x,y]
      endelse

    endfor

;    spawn, 'sleep 1'

  endfor

  return, cubeFinal




end
