
pro weightedCentroid, x, y, density, xbar, ybar
COMPILE_OPT IDL2, HIDDEN
  d = density

  mass = total(d)

  if mass eq 0 then begin
    mass = n_elements(d)
    d = 1.0
  endif

  xBar = total(x*d)/mass
  yBar = total(y*d)/mass

end

function binRoundness, x, y, pixelSize
COMPILE_OPT IDL2, HIDDEN

  n = n_elements(x)

  rEff = sqrt(n/!Pi)*pixelSize ; Rayon d'un disque de l'aire du pixel

  xbar = total(x)/n
  ybar = total(y)/n

  rMax = sqrt(max((x-xbar)^2 + (y-ybar)^2)) ; La distance maximale entr ele centre du pixel et le pixel le plus eloigne

  return, rMax/rEff -1
end

function createMap, x, y, mapInfo, pixelSize=pix
COMPILE_OPT IDL2

  n = n_elements(x)

  if n_elements(pix) eq 0 then begin
    pix = !values.f_infinity
    for i=0, n-2 do pix = min((x[i]-x[i+1:*])^2 + (y[i]-y[i+1:*])^2) < pix
  endif

  if pix le 0 then pix = 1

  minmaxx = minmax(x)
  minmaxy = minmax(y)
  sizex = (minmaxx[1] - minmaxx[0]) > 1
  sizey = (minmaxy[1] - minmaxy[0]) > 1

  ; Which minimum scale factor will we need?
  n = n_elements(x)
  test = lonarr(sizex/pix+1, sizey/pix+1)
  for i=0, n-1 do begin
    test[(x[i]-minmaxx[0])/pix, (y[i]-minmaxy[0])/pix] = test[(x[i]-minmaxx[0])/pix, (y[i]-minmaxy[0])/pix] + 1
  endfor

  scaleFactor = sqrt(max(test))
  print, 'Estimated scale factor is ' +strtrim(scaleFactor,2)
  test = 0

  if (sizex*sizey*scaleFactor*scaleFactor) gt 100000000 then return, -1

  tryAgain:

  widthx = ceil(scaleFactor*(sizex / pix))
  widthy = ceil(scaleFactor*(sizey / pix))

  scalex = widthx/sizex
  scaley = widthy/sizey

  map = lonarr(widthx+1, widthy+1) -1

  maxTransform = 0
  maxMultiplicator = 1

  multiplicatorxTable = (indgen(scaleFactor)+1) # replicate(1, scaleFactor)
  multiplicatoryTable = replicate(1, scaleFactor) # (indgen(scaleFactor)+1)

  for i=0, n-1 do begin
    transform = 0
    multiplicatorx = 1
    multiplicatory = 1
    reverse = 0
    while 1 do begin
      there:
      case transform of
        0: begin
          tx = round((x[i]-minmaxx[0])*scalex)
          ty = round((y[i]-minmaxy[0])*scaley)
        end
        1: begin
          tx = round((x[i]-minmaxx[0])*scalex)+1*multiplicatorx
          ty = round((y[i]-minmaxy[0])*scaley)
        end
        2: begin
          tx = round((x[i]-minmaxx[0])*scalex)
          ty = round((y[i]-minmaxy[0])*scaley)+1*multiplicatory
        end
        3: begin
          tx = round((x[i]-minmaxx[0])*scalex)-1*multiplicatorx
          ty = round((y[i]-minmaxy[0])*scaley)
        end
        4: begin
          tx = round((x[i]-minmaxx[0])*scalex)
          ty = round((y[i]-minmaxy[0])*scaley)-1*multiplicatory
        end
        5: begin
          tx = round((x[i]-minmaxx[0])*scalex)+1*multiplicatorx
          ty = round((y[i]-minmaxy[0])*scaley)+1*multiplicatory
        end
        6: begin
          tx = round((x[i]-minmaxx[0])*scalex)-1*multiplicatorx
          ty = round((y[i]-minmaxy[0])*scaley)-1*multiplicatory
        end
        7: begin
          tx = round((x[i]-minmaxx[0])*scalex)+1*multiplicatorx
          ty = round((y[i]-minmaxy[0])*scaley)-1*multiplicatory
        end
        8: begin
          tx = round((x[i]-minmaxx[0])*scalex)-1*multiplicatorx
          ty = round((y[i]-minmaxy[0])*scaley)+1*multiplicatory
        end
        9: begin
          if multiplicatory lt scaleFactor then begin
            if reverse eq 0 then begin
              if multiplicatorx eq multiplicatory then begin
                multiplicatorx = multiplicatorx+1
                multiplicatory = 1
              endif else begin 
                multiplicatory = multiplicatory+1
              endelse
              if multiplicatorx eq multiplicatory then reverse = 1
            endif else begin
              if multiplicatorx eq multiplicatory then begin
                multiplicatorx = 1
              endif else begin
                multiplicatorx = multiplicatorx+1
              endelse
              if multiplicatorx eq multiplicatory then reverse = 0
            endelse

;            multiplicatory = multiplicatory + ((multiplicatory lt multiplicatorx)?1:0)
;            multiplicatorx = multiplicatorx + ((multiplicatorx eq multiplicatory)?1:0)
   
;            print, multiplicatorx, multiplicatory

            maxMultiplicator = maxMultiplicator > multiplicatory
            transform = 0
            goto, there
          endif else begin
            ; We are unable to fit all bins in map...
            scaleFactor = scaleFactor*sqrt(2)
            print, 'Scaling up to ' + strtrim(scaleFactor, 2)
;            if scaleFactor gt 8 then begin
;              print, 'Scale factor reached 8! We stop befor bad things happen!'
;              error
;            endif
            goto, tryAgain
          endelse
        end
      endcase

      tx = (tx > 0) < widthx
      ty = (ty > 0) < widthy

      if map[tx,ty] eq -1 then begin
        map[tx,ty] = i
        break
      endif

      transform = transform + 1
      maxTransform = maxTransform > transform
    endwhile
  endfor

  mapInfo = fltarr(6)
  mapInfo[0] = minmaxx[0]
  mapInfo[1] = minmaxy[0]
  mapInfo[2] = scalex
  mapInfo[3] = scaley
  mapInfo[4] = widthx
  mapInfo[5] = widthy

  return, map

end


function mapLookup, map, x, y, radius, mapInfo
COMPILE_OPT IDL2
ON_ERROR, 2
  result = map[(x-mapInfo[0]-radius)*mapInfo[2]>0:(x-mapInfo[0]+radius)*mapInfo[2]<mapInfo[4], (y-mapInfo[1]-radius)*mapInfo[3]>0:(y-mapInfo[1]+radius)*mapInfo[3]<mapInfo[5]]
  if (n_elements(result) eq 1) and (result[0] eq -1) then return, -1

  real = where(result ne -1)
  
  if (n_elements(real) eq 1) and (real[0] eq -1) then return, -1
  return, result[real]
end


;pro newBinAccretion, x, y, signal2d, noise2d, cube, targetSn, class
  
;  sz = size(cube)
;  n = n_elements(x)

;  class2d = lonarr(sz[2], sz[3])
;  good2d = bytarr(sz[2], sz[3])

;  ; On definit quelques thresholds
;  pixelThreshold = 1.2^2
;  snThreshold = 0.8*targetSn
;  snDropThreshold = 0.2*targetSn

;  ; Pour les statistiques
;  snDropStatistics = 0
;  roundnessDropStatistics = 0
;  singlePixelBinStatistics = 0
;  multiplePixelBinStatistics = 0

;  ; Pour l'affichage du temps qui reste
;  printThreshold = n
;  printTime = systime(/seconds)
;  startTime = systime(/seconds)

;  sn = max(signal2d/noise2d, currentBin)
;  currentX = x[currentBin]
;  currentY = y[currentBin]

;  for ind=1, n-1 do begin
;    currentX = x[currentBin]
;    currentY = y[currentBin]
;    currentRaie = cube[*,currentX, currentY]

;    while 1 do begin
;      if computeSignalNoiseValue(currentRaie) ge snThreshold then begin
;        break
;      endif

;    endwhile

;    class2d[currentX, currentY] = ind




;  endfor


;  return, class
;end


pro binAccretion, x, y, signal, noise, cube, targetsn, class, pixelsize, p, tryhard
COMPILE_OPT IDL2, HIDDEN

  if n_elements(tryhard) eq 0 then tryhard = 0

  sz = size(cube)
  n = n_elements(x)

  class = lonarr(n)
  good = bytarr(n)

  zeros = where((signal eq 0.0) and (noise eq 1.0))
  if zeros[0] ne -1 then begin
    class[zeros] = 1
    n = n-n_elements(zeros)
  endif

  pixelSize=1.0

  ; On definit quelques thresholds
  pixelThreshold = (1.2*pixelSize)^2
  snThreshold = 0.8*targetSn
  snDropThresholdFactor = 0.3

  minmaxx = minmax(x)
  minmaxy = minmax(y)

  xsize = minmaxx[1]-minmaxx[0]
  ysize = minmaxy[1]-minmaxy[0]

  ; We construct a map that will tell us the pixel position in the array
  ; given it's x/y coordinates
  ; This map will avoid us computing uselessly every pixel distance from
  ; a given pixel when we are looking for a candidate pixel
  map = createMap(x, y, mapInfo, pixelSize=pixelSize)

  ; We beggin with the pixel with the highest s/n ratio

; MODIF optimisation
  sn = max(signal/noise, currentBin)

  print, 'First bin is at ' + strtrim(x[currentBin],2) + ', ' + strtrim(y[currentBin],2)

  distance = long(sqrt((x[currentBin]-x)^2+(y[currentBin]-y)^2))
  distance_max = max(distance)
  area = histogram(distance, binsize=1, reverse_indices=r_distance, locations=r_loc)
  distance_idx_max = max(r_loc)

  nextbindistance = 0
  nextbindistancestep = ceil(sqrt(1000.0/!pi))
  nextbinpool = r_distance[r_distance[nextbindistance]:r_distance[nextbindistance+nextbindistancestep]-1] ;where(distance lt nextbindistance)

;  unbinned = where(class eq 1)
;  distance[unbinned] = 1000000

  old_xbar = x[currentBin]
  old_ybar = y[currentBin]
  old_weight = signal[currentBin]

  binSizeLimit = 100*targetSn/(mean(signal/noise))
  print, 'BinSizeLimit is ', binSizeLimit

  if p eq 1 then im = tvrd() ;bytarr(sz[3], sz[2])+255

;  printThreshold = n
;  old_m = printThreshold
  printTime = systime(/seconds)
  startTime = systime(/seconds)
  snDropStatistics = 0
  roundnessDropStatistics = 0
  successfulBinStatistics = 0
  acceptedBad = lonarr(10)
  rejectedBad = lonarr(10)

  for ind=2, n-1 do begin
    ; We start by looking at most 10 pixels every side of the current bin
    ; If this is not enough, we will slowly raise this level
    binRadius = 10
    binThreshold = (binRadius^2)*0.5

    sn = computeSignalNoiseValue(cube[*, x[currentBin], y[currentBin]])
    realGood = currentBin
    realBad = -1
    bad = 0

    currentRaie = cube[*, x[currentBin], y[currentBin]]

    class[currentBin] = ind  ; Assign current bin a bin value
    xbar = x[currentBin]
    ybar = y[currentBin]

    old_m = 0
    m = 0

    while 1 do begin
      if (sn gt 0.8*targetSn) then begin ;/sqrt(sqrt(sqrt(float(n_elements(realGood)))))) then begin
        good[realGood] = 1
        modifiedPixels=realGood
        ; Put back bad pixels to the pixel pool...
        ; ******** A decommenter pour remettre
        ; les mauvais pixels dans le lot
        if (realBad[0] ne -1) and (tryhard ne 0) then begin
          good[realBad] = 0
          class[realBad] = 0
        endif
        ; ***************
        successfulBinStatistics = successfulBinStatistics + 1
        acceptedBad[bad<(n_elements(acceptedBad)-1)] = acceptedBad[bad<(n_elements(acceptedBad)-1)]+1
;        print, 'Binned ', sn
        break
      endif

      if m lt binThreshold then begin
        possibleUnbinned = mapLookup(map, floor(xbar), floor(ybar), binRadius, mapInfo)
      endif

      unbinnedBins = where(class[possibleUnbinned] eq 0, m)

      ; Make sure we have enough unbinned pixels in the viciny...
      if m lt binThreshold then begin
;        if m gt old_m then begin ; Si on n'a pas fait de gain en pixels en augmentant
                                 ; le rayon de recherche, c'est qu'il 
                                 ; n'y a pas plus de pixels de disponibles
;          if ((binRadius lt xsize) and (binRadius lt ysize)) then begin
          if ((binRadius lt 40) and (binRadius lt xsize) and (binRadius lt ysize)) then begin
            old_m = m
            m = 0
            binRadius = binRadius*4
            binThreshold = (binRadius^2)*0.5
;            print, 'binRadius raised to ', binRadius
            continue
          endif
;        endif
;        print, "Incrementing binRadius didn't change a thing..."
      endif

      if m eq 0 then begin
        print, 'No more pixels!'
        break
      endif

      unbinned = possibleUnbinned[unbinnedBins]

      ; Find the closest pixel to the current bin
      minDist = min((x[unbinned]-xbar)^2 + (y[unbinned]-ybar)^2, k)
      minDist = min((x[currentBin]-x[unbinned[k]])^2 + (y[currentBin]-y[unbinned[k]])^2)
;      minDist = (x[unbinned[k]] - xbar)^2 + (y[unbinned[k]] - ybar)^2

      ; Estimate now the bin roundness...
      tryPixel = unbinned[k]

      ; Si le nouveau pixel a lui seul a un SN suffisant,
      ; on le sort du bin pour l'instant et il sera rebinne tout seul
      ; a une autre passe
      ; ************** A decommenter pour
      ; remettre les mauvais pixels dans le lot
;      if signal[tryPixel]/noise[tryPixel] ge 0.8*targetSn then begin
;        print, 'Single pixel with SN high enough'
;        if realBad[0] eq -1 then begin
;         realBad = tryPixel
;        endif else begin
;          realBad = [realBad, tryPixel]
;        endelse 
;        class[tryPixel] = ind
;        continue
;      endif
      ; *************

      nextBin = [currentbin, tryPixel]
      roundness = binRoundness(x[nextBin], y[nextBin], pixelSize)

      snOld = sn

      currentRaie = currentRaie + cube[*, x[tryPixel], y[tryPixel]]

;     le total(total(,2),2) n'est PAS VALIDE!!!!!!!!!
;      sn = computeSignalNoiseValue(total(total(cube[*, x[nextBin], y[nextBin]], 2), 2))

      sn = computeSignalNoiseValue(currentRaie)
;        sn = computeSignalNoiseValue(total(total(cube[*, x[nextBin], y[nextBin]], 2), 2), _signal, _noise, _flux, _continum, _b, _start, _stop, _raiet, _dispersion)
;        plot, findgen(sz[1]), total(total(cube[*, x[nextBin], y[nextBin]], 2), 2)
;        xyouts,5, max(total(total(cube[*, x[nextBin], y[nextBin]], 2), 2)), 'S/N: ' + strtrim(sn,2)
;        oplot, findgen(sz[1]), replicate(_continum, sz[1])
;        oplot, [_b[0]], [_b[1]+_continum], psym=1
;        oplot, replicate(_start, 2), [0,max(total(total(cube[*, x[nextBin], y[nextBin]], 2), 2))]
;        oplot, replicate(_stop, 2), [0,max(total(total(cube[*, x[nextBin], y[nextBin]], 2), 2))]

;        print, sn, _signal, _noise, _dispersion, _continum, _b[1]

;        cursor, tempx, tempy, /device, /up

      ; On avorte le bin s'il n'est pas beau (Le voronoi Arien!)
      if((minDist gt pixelThreshold) or (roundness gt 0.3) or (n_elements(nextBin) gt binSizeLimit)) then begin
;       bad_bin:
        roundnessDropStatistics = roundnessDropStatistics + 1
        rejectedBad[bad<(n_elements(rejectedBad)-1)] = rejectedBad[bad<(n_elements(rejectedBad)-1)]+1

        ; **************** A decommenter pour remettre les pixels
        if (n_elements(nextBin) gt binSizeLimit) and (tryhard ne 0) then begin
;          print, 'Bin size limit reached'
          class[nextBin[1:n_elements(nextBin)-1]] = 0
          if realBad[0] ne -1 then begin
            if n_elements(realGood) gt 1 then begin
              realBad = [realBad, realGood[1:(n_elements(realGood)-1)>1]]
            endif
            class[realBad] = 0
          endif else begin
            if n_elements(realGood) gt 1 then begin
              realBad = realGood[1:(n_elements(realGood)-1)>1]
              class[realBad] = 0
            endif
          endelse
          realGood = realGood[0]
          realGood=0
;        ; ****************
       endif

        ; On remet tous les pixels de ce bin
        ; dans le pool de pixels disponibles sauf le premier
        modifiedPixels = nextBin[0]
        ; ************ A decommenter pour remettre les pixels
        if tryhard ne 0 then begin
          class[nextBin[1:n_elements(nextBin)-1]] = 0
          if realBad[0] ne -1 then begin
            if n_elements(realGood) gt 1 then begin
              realBad = [realBad, realGood[1:(n_elements(realGood)-1)>1]]
            endif
            class[realBad] = 0
          endif else begin
            if n_elements(realGood) gt 1 then begin
              realBad = realGood[1:(n_elements(realGood)-1)>1]
              class[realBad] = 0
            endif
          endelse
          realGood = realGood[0]
          realGood=0
        endif

        ; ****************
        break
      endif

;      print, snOld, sn

      if sn lt (snOld-snOld*snDropThresholdFactor) then begin
        ; Ce n'est pas un bon pixel... on l'enleve du bin courant...
        nextBin = nextBin[0:n_elements(nextBin)-2]
        snDropStatistics = snDropStatistics + 1
        bad = bad + 1
        currentRaie = currentRaie - cube[*, x[tryPixel], y[tryPixel]]
        if realBad[0] eq -1 then begin
          realBad = tryPixel
        endif else begin
          realBad = [realBad, tryPixel]
        endelse
;        print, 'Dropped'
      endif else begin
        realGood = [realGood, tryPixel]
;        print, 'Good'
      endelse
      

      ; On ajoute le pixel candidat au bin pour ne plus repasser dessus
      ; Et on continue...
      class[tryPixel] = ind
;      distance[tryPixel] = 1000000
      currentBin = nextBin

      ; Trouve le nouveau centroid
      weightedCentroid, x[currentBin], y[currentBin], signal[currentBin], xBar, yBar

    endwhile

    next_bin:

    if p eq 1 then begin
      im[x[modifiedPixels], y[modifiedPixels]]=0
  ;    im[x[binned], y[binned]] = 0
  ;    tv, im
    
      tv, im[(min(x[modifiedPixels])-1)>0:(max(x[modifiedPixels])+1)<(sz[2]-1), (min(y[modifiedPixels])-1)>0:(max(y[modifiedPixels])+1)<(sz[3]-1)], (min(x[modifiedPixels])-1)>0, (min(y[modifiedPixels])-1)>0
    endif

;    if m lt printThreshold then begin
     
      timeTemp = systime(/seconds)

    if timeTemp gt printTime then begin
;      pixsec = (old_m-m)/(time_temp-print_time)
      toto = where(class eq 0, m)
      pixsec = (n-m)/(timeTemp-startTime)
      print, '    ' + strtrim(m,2) + ' pixels left, ' + strtrim(pixsec,2) + ' pixels/second. Successfully binned: ' + strtrim(n_elements(where(good eq 1)),2)
      print, '      Eslapsed time: ' + strtrim((timeTemp-startTime)/60.0,2) + ' minutes'
;      print, '      Remaining time: ' + strtrim(m/(pixsec*60),2) + ' minutes'
      print, '      ' + strtrim(float(n-m)/float(n)*100,2) + ' percent done'
      print, '      Good: ' + strtrim(successfulBinStatistics,2) + ', bad: ' + strtrim(roundnessDropStatistics,2) + ' ' + strtrim(snDropStatistics,2)
      print, '      Accepted: ', acceptedBad[0:5] ;, total(acceptedBad[6:n_elements(acceptedBad)-1])
      print, '      Rejected: ', rejectedBad[0:5] ;, total(rejectedBad[6:n_elements(rejectedBad)-1])
;      old_m = m
      printTime = timeTemp + 10
;      printThreshold = m - 1000
    endif

;    endif

    ; Continue with the closest pixel to
    ; the centroid of all the currently binned pixels

; MODIF optimisation
;    weightedCentroid, [x[modifiedPixels], old_xBar], [y[modifiedPixels], old_yBar], [signal[modifiedPixels], old_weight], xBar, yBar
;    weightedCentroid, [x[modifiedPixels], old_xBar], [y[modifiedPixels], old_yBar], 0, xBar, yBar
   
    old_xBar = xBar
    old_yBar = yBar
; MODIF optimisation
;    old_weight = old_weight + total(signal[realGood])



;    weightedCentroid, x[binned], y[binned], signal[binned], xBar, yBar

;    minDist = min(((x[unbinned]-xbar)^2 + (y[unbinned]-ybar)^2)*(signal[unbinned]/noise[unbinned]), k)
;    distance = (x[unbinned]-xbar)^2 + (y[unbinned]-ybar)^2)
;    order = sort(distance)
;    idx = (reverse(sort))[0:10<(n_elements(sort)-1)]
;    minDist = min(distance[unbinned[idx]]*signal[unbinned[idx]]/noise[unbinned[idx]], k)
;    k = idx[k]
    
    
; MODIF optimisation    
;    minDist = min(((x[unbinned]-xbar)^2 + (y[unbinned]-ybar)^2), k)

;    unbinned = where(class eq 0, COMPLEMENT=binned, m)
  
;    if m eq 0 then break  ; All pixels are binned

    unbinned = -1

    while unbinned[0] eq -1 do begin
      if nextbinpool[0] ne -1 then unbinned = where(class[nextbinpool] eq 0)

      if (unbinned[0] eq -1) and (nextbindistance le distance_idx_max) then begin
;        print, 'Increasing next bin distance'
        nextbindistance += nextbindistancestep
        nextbindistancestep = ceil(sqrt((1000.0+nextbindistance^2*!pi)/!pi)) - nextbindistance
        nextbindistancestep <= (distance_idx_max - nextbindistance)
        if nextbindistancestep le 0 then break
        nextbinpool = r_distance[r_distance[nextbindistance]:r_distance[nextbindistance+nextbindistancestep]-1]
        temp = where(class[nextbinpool] eq 0)
        if temp[0] ne -1 then nextbinpool = nextbinpool[temp]
;where((distance lt nextbindistance) and (class eq 0))
      endif

      if (unbinned[0] eq -1) and (nextbindistance gt max(distance)) then break
    endwhile

    if unbinned[0] eq -1 then break

    minDist = min(distance[nextbinpool[unbinned]], k)

    currentBin = nextbinpool[unBinned[k]]
  endfor

  class = class * good
  test = where(class eq 0, m)
  print, strtrim(m, 2) + ' pixels left alone...'

end


pro reassignBadBins, x, y, signal, noise, targetsn, class, xnode, ynode
COMPILE_OPT IDL2, HIDDEN

  area = histogram(class, reverse_indices=r, min=1)
  good = where(area gt 0, nnodes)

  xnode = fltarr(nnodes)
  ynode = fltarr(nnodes)

  print, 'First pass'
  for i=0, nnodes-1 do begin
    k=good[i]
    p = r[r[k]:r[k+1]-1]
    weightedCentroid, x[p], y[p], signal[p], xBar, yBar
    xnode[i] = xBar
    ynode[i] = yBar
  endfor

  print, 'Second pass'
  bad = where(class eq 0, m)

  map = createMap([xnode, minmax(x)], [ynode, minmax(y)], mapInfo, pixelSize=1.0)
  binradius=20
  increment = long(max(class)) + 1
  good = [good, increment, increment+1]
  increment += 2

  ; Le truc ici, c'est qu'on cree des bins de bad pixels s'ils sont isoles...
  for i=0, m-1 do begin
    possible = mapLookup(map, floor(x[bad[i]]), floor(y[bad[i]]), binRadius, mapInfo)
    
    if possible[0] eq -1 then begin
      class[bad[i]] = increment
      increment = increment + 1
      continue
    endif

    tmp = min((x[bad[i]]-xnode[possible])^2 + (y[bad[i]]-ynode[possible])^2, index)
    class[bad[i]] = good[possible[index]] + 1
  endfor


  print, 'Third pass'
  area = histogram(class, reverse_indices=r)
  good = where(area gt 0, nnodes)

  xxnode = fltarr(nnodes)
  yynode = fltarr(nnodes)

  xxnode[0:n_elements(xnode)-1] = xnode
  yynode[0:n_elements(ynode)-1] = ynode
  xnode = xxnode
  ynode = yynode

  for i=0, nnodes-1 do begin
    k = good[i]
    p = r[r[k]:r[k+1]-1]
    weightedCentroid, x[p], y[p], signal[p], xBar, yBar
    xnode[i] = xBar
    ynode[i] = yBar
  endfor

end


pro cvtEqualMass, x, y, signal, noise, class, xnode, ynode, pixelSize, iter
COMPILE_OPT IDL2, HIDDEN

  nbins = n_elements(xnode)
  nbpixels = n_elements(signal)
  class = lonarr(nbpixels)
  dens = (signal/noise)^2

  iter = 1

  sizex = max(x)-min(x)
  sizey = max(y)-min(y)

  repeat begin
    xnodeOld = xnode
    ynodeOld = ynode

    minx = round(min(xnode))
    miny = round(min(ynode))

    map = createMap(xnode, ynode, mapInfo, pixelSize = pixelSize)

    for i=0, nbpixels-1 do begin
      trysize = 5
      while 1 do begin
        candidates = mapLookup(map, x[i], y[i], trySize, mapInfo)

        if n_elements(candidates) ne 1 then break
        if candidates[0] ne -1 then break

        trysize = trysize*2
      endwhile

      tmp = min((x[i] - xnode[candidates])^2 + (y[i] - ynode[candidates])^2, index)
      class[i] = candidates[index]

;      tmp = MIN((x[i]-xnode)^2+(y[i]-ynode)^2, index)
;      class[i] = index

    endfor

    area = histogram(class, reverse_indices=r)

    for i=0, nbins-1 do begin
      if r[i] ne r[i+1] then begin
        index = r[r[i]:r[i+1]-1]  ; find subscripts of pixels in bin i
      endif else begin
        index = r[r[i]]
      endelse

      weightedCentroid, x[index], y[index], dens[index]^2, xb, yb
      xnode[i] = xb
      ynode[i] = yb
    endfor

    diff = total((xnode-xnodeOld)^2 + (ynode-ynodeOld)^2)
    iter = iter + 1

  endrep until diff/nbins lt 0.01

end

pro computeUsefulBinQuantities, x, y, signal, noise, cube, class, xbar, ybar, sn, sn2d, area, binSize
COMPILE_OPT IDL2

  area = histogram(class, reverse_indices=r)
  nbins = n_elements(area)
  xbar = fltarr(nbins)
  ybar = fltarr(nbins)
  sn = fltarr(nbins)

  sz = size(cube)
  binSize = fltarr(sz[2], sz[3])

  minx = min(x)
  miny = min(y)

  sn2d = fltarr(max(x)-min(x)+1, max(y)-min(y)+1)

  print, 'Nbins: ', nbins
  help, r

  for i=0, nbins-1 do begin
    index = r[r[i]:r[i+1]-1]
    weightedCentroid, x[index], y[index], signal[index], xb, yb
    xbar[i] = xb
    ybar[i] = yb
    binSize[x[index], y[index]] = float(n_elements(index))
    if n_elements(index) eq 1 then begin
      sn[i] = computeSignalNoiseValue(cube[*, x[index], y[index]])
    endif else begin
      raieTemp = cube[*, x[index[0]], y[index[0]]]

      for j=1, n_elements(index)-1 do raieTemp = raieTemp + cube[*, x[index[j]], y[index[j]]]

      sn[i] = computeSignalNoiseValue(raieTemp)
    endelse

    sn2d[x[index]-minx, y[index]-miny] = sn[i]
  endfor

end

pro stripUnusedBins, class
COMPILE_OPT IDL2, HIDDEN

  area = histogram(class, reverse_indices=r)
  good = where(area gt 0, nbins)

  for i=0, nbins-1 do begin
    g = good[i]
    ind = r[r[g]:r[g+1]-1]
    class[ind] = i
  endfor

end

pro displayPixels, x, y, counts, pixelSize
COMPILE_OPT IDL2

  ; Plots colored pixels with the same pixels size

  plot, [min(x)-pixelSize, max(x)+pixelSize], [min(y)-pixelSize, max(y)+pixelSize], /NODATA, /XSTYLE, /YSTYLE, XTITLE='Pixel', YTITLE='Pixel', /iso ; modification olivier Pixel
;  plots, [min(x)-pixelSize, max(x)+pixelSize], [min(y)-pixelSize, max(y)+pixelSize] ; modification olivier Pixel
  countMax = max(counts, MIN=countMin)
  countRange = countMax - countMin
  for j=0, n_elements(counts)-1 DO BEGIN
    xprime = x[j] + [-0.5, -0.5, +0.5, +0.5, -0.5] * pixelSize
    yprime = y[j] + [+0.5, -0.5, -0.5, +0.5, +0.5] * pixelSize
    polyfill, xprime, yprime,  COLOR=FLOAT(counts[j]-countMin)/countRange*255;, /line_fill
  endfor
end


function createFinalCube, cube, cal_valid, x, y, xnode, ynode, class, class2d, cubeBinned, region=reg
COMPILE_OPT IDL2, HIDDEN
  sz = size(cube)

  cubeSmooth = fltarr(sz[1], sz[2], sz[3])
  outRaies = fltarr(sz[1], max(class)-min(class)+1)
  class2d = lonarr(sz[2], sz[3])
  n = lonarr(max(class)-min(class)+1)

  ; TODO: Optimiser la boucle suivante
  for i=0, n_elements(class)-1 do begin
    outRaies[*, class[i]] = outraies[*, class[i]] + cube[*, x[i], y[i]]
    class2d[x[i], y[i]] = class[i]
    n[class[i]] = n[class[i]] + 1
  endfor

  for i=0, n_elements(n)-1 do begin
    outRaies[*,i] = outraies[*,i]/n[i]
  endfor

  cubeBinned = fltarr(sz[1], sz[2], sz[3])
  m = minmax(class)
  area = histogram(class, reverse_indices=r)

  for i=m[0], m[1] do begin
;    idx = where(class eq i)
    idx = r[r[i]:r[i+1]-1]
    if idx[0] ne -1 then begin
;      cubeBinned[*,idx] = outraies[*, i]
;    endif
      for j=0, n_elements(idx)-1 do begin
        cubeBinned[*,x[idx[j]],y[idx[j]]] = outraies[*, class[idx[j]]]
      endfor
    endif
  endfor

  cubeBinned = reform(cubeBinned, sz[1], sz[2], sz[3])

;  cubeSmooth = smoothBinnedCube(cubeBinned, class2d, cal_valid=cal_valid, region=reg)

  triangulate, xNode, yNode, triangles, b
  
  xmin = floor(min(xNode))>0
  xmax = ceil(max(xNode))<(sz[2]-1)
  ymin = floor(min(yNode))>0
  ymax = ceil(max(yNode))<(sz[3]-1)

  for i=0, sz[1]-1 do begin
    cubeSmooth[i, xmin:xmax, ymin:ymax] = trigrid(xnode, ynode, outraies[i, *], triangles, nx=xmax-xmin+1, ny=ymax-ymin+1)
  endfor

  return, cubeSmooth
end

;+
; NAME:
;        voronoi_3d_binning
;
; AUTHOR:
;        Olivier Daigle (odaigle@astro.umontreal.ca)
;
; PURPOSE:
;        Execute un lissage sur un cube de donnees
; 
; INPUTS:
;           cube: le cube de donnees a traiter. C'est d'apres ce cube que le
;                 signal/bruit sera recalcule apres chaque binning
;       targetsn: le rapport s/b voulu
;
; INPUT facultatifs:
;              x: La position en X des pixels
;              y: La position en Y des pixels
;                 Les pixels d'entree doivent etre a des positions
;                 entieres et avoir une grosseur de 1
;                 Tous les pixels doivent etre presents
;         signal: une carte 1d de la valeur du signal dans chaque pixel
;          noise: une carte 1d de la valeur du bruit dans chaque pixel
;       signal2d: une carte 2d de la valeur du signal dans chaque
;                 pixel
;        noise2d: une carte 2d de la valeur du bruit dans chaque pixel
;
; OUTPUTS:
;          class: class[i] = numero du bin du pixel x[i], y[i]
;          xnode: xnode[i] = position en x du centre de masse du bin de
;                 classe i
;          ynode: ynode[i] = position en y du centre de masse du bin
;                 de class i
;           xbar: 
;           ybar:
;             sn: sn[i] = rapport s/n du bin de classe i
;           area:

function voronoi_3d_fast, cube, cal_valid, targetSN, x, y, signal, noise, signal2d, noise2d, class, class2d, xnode, ynode, xbar, ybar, sn, sn2d, area, binA, cubeBinned, binSize, p, region=reg, tryhard=tryhard
COMPILE_OPT IDL2

  szc = size(cube)

  if ((n_elements(noise2d) eq szc[2]*szc[3]) and (n_elements(signal2d) eq szc[2]*szc[3])) then begin
    x = fltarr(n_elements(signal2d), /nozero)
    y = fltarr(n_elements(signal2d), /nozero)
    signal = fltarr(n_elements(signal2d), /nozero)
    noise = fltarr(n_elements(signal2d), /nozero)

    sz = size(signal2d)
    xsize = sz[2]
    ysize = sz[3]

    for i=0, ysize-1 do begin
      x[i*xsize:i*xsize+xsize-1] = findgen(xsize)
      y[i*xsize:i*xsize+xsize-1] = i

      signal[i*xsize:i*xsize+xsize-1] = signal2d[*,i]
      noise[i*xsize:i*xsize+xsize-1] = noise2d[*,i]
    endfor
  endif
   
  if (n_elements(x) eq 0) or (n_elements(y) eq 0) or (n_elements(signal) eq 0) or (n_elements(noise) eq 0) then begin
    xsize = szc[2]
    ysize = szc[3]

    x = fltarr(xsize*ysize, /nozero)
    y = fltarr(xsize*ysize, /nozero)
    signal = fltarr(xsize*ysize, /nozero)
    noise = fltarr(xsize*ysize, /nozero)

    signal2d = fltarr(xsize, ysize, /nozero)
    noise2d = fltarr(xsize, ysize, /nozero)

    for i=0, ysize-1 do begin
      x[i*xsize:i*xsize+xsize-1] = findgen(xsize)
      y[i*xsize:i*xsize+xsize-1] = i
    endfor

    for i=0, xsize*ysize-1 do begin
      sn = computeSignalNoiseValue(cube[*, x[i], y[i]], s, n)
      signal[i] = s
      noise[i] = n

      signal2d[x[i], y[i]] = s
      noise2d[x[i], y[i]] = n
    endfor

  endif

  goto, end_remove
  ; Remove useless pixels...
  noZero = where(signal ne 0)
  if (n_elements(noZero) ne 1) or (noZero[0] ne -1) then begin
   nn = n_elements(signal)
    signal = signal[noZero]
    noise = noise[noZero]
    x = x[noZero]
    y = y[noZero]

    print, '  Removing ' + strtrim(nn-n_elements(noZero), 2) + ' 0-value signal pixels'
    print, '  Leaving ' + strtrim(n_elements(noZero), 2) + ' pixels out of ' + strtrim(nn, 2)
  endif

  end_remove:

  ; Pseudo-paranoya
  testZero = where(noise eq 0)
  if (n_elements(testZero) ne 1) or (testZero[0] ne -1) then begin
    signal[testZero] = 0
    noise[testZero] = 1
  endif

  testZero = where(noise2d eq 0)
  if (n_elements(testZero) ne 1) or (testZero[0] ne -1) then begin
    signal2d[testZero] = 0
    noise2d[testZero] = 1
  endif

  print, 'Signal: ', mean(signal), ', ', sqrt(stddev(signal))
  print, 'Noise: ', mean(noise), ', ', sqrt(stddev(noise))
  print, 'S/N:', mean(signal/noise), ', ', sqrt(stddev(signal/noise))

  if p eq 1 then begin
    window, 1, xsize=szc[2], ysize=szc[3]
    loadct, 5, /silent
    tvscl, rescale(signal2d/noise2d, 0.01, 0.99)
  endif

  print, '  Bin accretion'
  binAccretion, x, y, signal, noise, cube, targetsn, class, pixelsize, p, tryhard
  binA = class

  print, '  Reassigning bad bins...'
  reassignBadBins, x, y, signal, noise, targetsn, class, xnode, ynode

  print, '   done!'
;  displayPixels, x, y, rnd[class], pixelSize

;  stripUnusedBins, class
;  print, '    done!'
;  print, '  Modified Lloyd algorythm...'
;  cvtEqualMass, x, y, signal, noise, class, xnode, ynode, pixelSize, iter
  print, '  Stripping unused bins...'
  stripUnusedBins, class
  print, '    done!'

  print, '  Compute useful bin quantities...'
  computeUsefulBinQuantities, x, y, signal, noise, cube, class, xbar, ybar, sn, sn2d, area, binSize
  print, '    done!'
;  stripUnusedBins, class

;  error
  w1 = where(area eq 1, complement=w2, m)
  print, 'Fractional S/N scatter (%):', stddev(sn[w2]-targetSN)/targetSN*100
  std = stddev(sn[w2]-targetSN)

  ; Plot...
  if p eq 1 then begin
    window, 2, xsize=szc[2], ysize=szc[3]
    loadct, 3, /silent
    rnd = randomu(seed, n_elements(xnode))
    displayPixels, x, y, rnd[class], 1.0
    oplot, xnode, ynode, psym=1, symsize=0.5
    rad = sqrt(xbar^2+ybar^2)

    window, 3
    loadct, 0, /silent
    toto = rescale(sn[w2], 0.01, 0.99, dmin, dmax)
    plot, rad[w2], sn[w2], psym=6, symsize=0.5, xtitle="R (pixels)", ytitle="Bin S/N", xrange=[min(rad), max(rad)], yrange=[0, dmax]
    plots, [min(rad), max(rad)], [targetSN, targetSN]
    plots, [min(rad), max(rad)], [targetSN-std, targetSN-std]
    plots, [min(rad), max(rad)], [targetSN+std, targetSN+std]
  endif

  print, '   Delaunay triangulating...'
  return, createFinalCube(cube, cal_valid, x, y, xnode, ynode, class, class2d, cubeBinned, region=reg)

end
